@@ -6,11 +6,13 @@
  */
 #include <config.h>
 #include <gnome.h>
+#include <locale.h>
 #include "gnumeric.h"
 #include "gnumeric-sheet.h"
 #include "eval.h"
 #include "format.h"
 #include "color.h"
+#include "cursors.h"
 
 static int         redraws_frozen = 0;
 static GHashTable *cell_hash_queue;
@@ -24,6 +26,16 @@ cell_formula_changed (Cell *cell)
 	cell_queue_recalc (cell);
 }
 
+static inline void
+cell_modified (Cell *cell)
+{
+	Sheet *sheet = cell->sheet;
+
+	/* Cells from the clipboard do not have a sheet attached */
+	if (sheet)
+		sheet->modified = TRUE;
+}
+
 void
 cell_set_formula (Cell *cell, char *text)
 {
@@ -33,15 +45,15 @@ cell_set_formula (Cell *cell, char *text)
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (text != NULL);
 
+	cell_modified (cell);
 	cell->parsed_node = expr_parse_string (&text [1],
+					       cell->sheet,
 					       cell->col->pos,
 					       cell->row->pos,
 					       &desired_format,
 					       &error_msg);
 	if (cell->parsed_node == NULL){
-		if (cell->text)
-			string_unref_ptr (&cell->text);
-		cell->text = string_get (error_msg);
+		cell_set_rendered_text (cell, error_msg);
 		return;
 	}
 	if (desired_format && strcmp (cell->style->format->format, "General") == 0){
@@ -74,6 +86,8 @@ cell_set_alignment (Cell *cell, int halign, int valign, int orient, int auto_ret
 	    (cell->style->orientation == orient))
 		return;
 
+	cell_modified (cell);
+
 	cell_queue_redraw (cell);
 	
 	cell->style->halign = halign;
@@ -94,6 +108,8 @@ cell_set_halign (Cell *cell, StyleHAlignFlags halign)
 	if (((unsigned int)cell->style->halign) == ((unsigned int) halign))
 		return;
 
+	cell_modified (cell);
+
 	cell_queue_redraw (cell);
 	cell->style->halign = halign;
 
@@ -107,6 +123,8 @@ cell_set_font_from_style (Cell *cell, StyleFont *style_font)
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (style_font != NULL);
 
+	cell_modified (cell);
+
 	cell_queue_redraw (cell);
 	
 	style_font_unref (cell->style->font);
@@ -139,6 +157,8 @@ cell_set_style (Cell *cell, Style *reference_style)
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (reference_style != NULL);
 
+	cell_modified (cell);
+
 	cell_queue_redraw (cell);
 	style_destroy (cell->style);
 	cell->style = style_duplicate (reference_style);
@@ -148,10 +168,207 @@ cell_set_style (Cell *cell, Style *reference_style)
 	cell_queue_redraw (cell);
 }
 
+void
+cell_comment_destroy (Cell *cell)
+{
+	CellComment *comment;
+	GList *l;
+	
+	g_return_if_fail (cell != NULL);
+
+	comment = cell->comment;
+	if (!comment)
+		return;
+	cell->comment = NULL;
+
+	/* Free resources */
+	string_unref (comment->comment);
+
+	if (comment->timer_tag != -1)
+		gtk_timeout_remove (comment->timer_tag);
+
+	if (comment->window)
+		gtk_object_destroy (GTK_OBJECT (comment->window));
+	
+	for (l = comment->realized_list; l; l = l->next)
+		gtk_object_destroy (l->data);
+	
+	g_free (comment);
+}
+
+static void
+cell_comment_cancel_timer (Cell *cell)
+{
+	if (cell->comment->timer_tag != -1){
+		gtk_timeout_remove (cell->comment->timer_tag);
+		cell->comment->timer_tag = -1;
+	}
+		
+}
+      
+static void
+cell_display_comment (Cell *cell)
+{
+	GtkWidget *window, *label;
+	int x, y;
+	
+	g_return_if_fail (cell != NULL);
+
+	cell_comment_cancel_timer (cell);
+	
+	window = gtk_window_new (GTK_WINDOW_POPUP);
+	label = gtk_label_new (cell->comment->comment->str);
+	gtk_container_add (GTK_CONTAINER (window), label);
+
+	gdk_window_get_pointer (NULL, &x, &y, NULL);
+	gtk_widget_set_uposition (window, x+10, y+10);
+
+	gtk_widget_show_all (window);
+
+	cell->comment->window = window;
+}
+
+static gint
+cell_popup_comment (gpointer data)
+{
+	Cell *cell = data;
+	
+	cell->comment->timer_tag = -1;
+
+	cell_display_comment (cell);
+	return FALSE;
+}
+
+static int
+cell_comment_clicked (GnomeCanvasItem *item, GdkEvent *event, Cell *cell)
+{
+	GnomeCanvas *canvas = item->canvas;
+
+	switch (event->type){
+	case GDK_BUTTON_RELEASE:
+		if (event->button.button != 1)
+			return FALSE;
+		if (cell->comment->window)
+			return FALSE;
+		cell_display_comment (cell);
+		break;
+
+	case GDK_BUTTON_PRESS:
+		if (event->button.button != 1)
+			return FALSE;
+		break;
+		
+	case GDK_ENTER_NOTIFY:
+		cell->comment->timer_tag = gtk_timeout_add (1000, cell_popup_comment, cell);
+		cursor_set_widget (canvas, GNUMERIC_CURSOR_ARROW);
+		break;
+
+	case GDK_LEAVE_NOTIFY:
+		cell_comment_cancel_timer (cell);
+		if (cell->comment->window){
+			gtk_object_destroy (GTK_OBJECT (cell->comment->window));
+			cell->comment->window = NULL;
+		}
+		break;
+		
+	default:
+		return FALSE;
+	}
+	return TRUE;
+}
+
+static void
+cell_comment_realize (Cell *cell)
+{
+	GList *l;
+
+	g_return_if_fail (cell->comment != NULL);
+
+	sheet_cell_comment_link (cell);
+	for (l = ((Sheet *)cell->sheet)->sheet_views; l; l = l->next){
+		SheetView *sheet_view = SHEET_VIEW (l->data);
+		GnomeCanvasItem *o;
+		
+		o = sheet_view_comment_create_marker (
+			sheet_view, 
+			cell->col->pos, cell->row->pos);
+
+		cell->comment->realized_list = g_list_prepend (
+			cell->comment->realized_list, o);
+
+		gtk_signal_connect (GTK_OBJECT (o), "event",
+				    GTK_SIGNAL_FUNC (cell_comment_clicked), cell);
+	}
+}
+
+static void
+cell_comment_unrealize (Cell *cell)
+{
+	GList *l;
+
+	g_return_if_fail (cell->comment != NULL);
+
+	sheet_cell_comment_unlink (cell);
+	for (l = cell->comment->realized_list; l; l = l->next){
+		GnomeCanvasItem *o = l->data;
+
+		printf ("Destruyendo: %p\n", o);
+		gtk_object_destroy (GTK_OBJECT (o));
+	}
+	g_list_free (cell->comment->realized_list);
+	cell->comment->realized_list = NULL;
+}
+
+void
+cell_realize (Cell *cell)
+{
+	g_return_if_fail (cell != NULL);
+
+	if (cell->comment)
+		cell_comment_realize (cell);
+}
+
+void
+cell_unrealize (Cell *cell)
+{
+	g_return_if_fail (cell != NULL);
+
+	if (cell->comment)
+		cell_comment_unrealize (cell);
+}
+
+void
+cell_set_comment (Cell *cell, char *str)
+{
+	int had_comments = FALSE;
+	
+	g_return_if_fail (cell != NULL);
+	g_return_if_fail (str != NULL);
+
+	cell_modified (cell);
+
+	cell_comment_destroy (cell);
+
+	cell->comment = g_new (CellComment, 1);
+	cell->comment->realized_list = NULL;
+	cell->comment->timer_tag = -1;
+	cell->comment->window = NULL;
+	
+	cell->comment->comment = string_get (str);
+
+	if (had_comments)
+		cell_queue_redraw (cell);
+
+	if (cell->sheet)
+		cell_comment_realize (cell);
+}
+
 void
 cell_set_foreground (Cell *cell, gushort red, gushort green, gushort blue)
 {
 	g_return_if_fail (cell != NULL);
+
+	cell_modified (cell);
 	
 	if (cell->style->valid_flags & STYLE_FORE_COLOR)
 		style_color_unref (cell->style->fore_color);
@@ -167,6 +384,8 @@ cell_set_background (Cell *cell, gushort red, gushort green, gushort blue)
 {
 	g_return_if_fail (cell != NULL);
 	
+	cell_modified (cell);
+
 	if (cell->style->valid_flags & STYLE_BACK_COLOR)
 		style_color_unref (cell->style->back_color);
 
@@ -182,6 +401,8 @@ cell_set_pattern (Cell *cell, int pattern)
 {
 	g_return_if_fail (cell != NULL);
 
+	cell_modified (cell);
+
 	cell->style->valid_flags |= STYLE_PATTERN;
 	cell->style->pattern = pattern;
 
@@ -202,6 +423,8 @@ cell_set_rendered_text (Cell *cell, char *rendered_text)
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (rendered_text != NULL);
 	
+	cell_modified (cell);
+
 	if (cell->text)
 		string_unref (cell->text);
 
@@ -252,18 +475,17 @@ cell_render_value (Cell *cell)
 void
 cell_set_text_simple (Cell *cell, char *text)
 {
+	struct lconv *lconv;
+	
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (text != NULL);
 
-	/* The value entered */
-	if (cell->entered_text)
-		string_unref (cell->entered_text);
+	cell_modified (cell);
 
 	if (cell->value){
 		value_release (cell->value);
 		cell->value = NULL;
 	}
-	cell->entered_text = string_get (text);
 	
 	if (cell->parsed_node){
 		sheet_cell_formula_unlink (cell);
@@ -277,10 +499,13 @@ cell_set_text_simple (Cell *cell, char *text)
 	} else {
 		Value *v = g_new (Value, 1);
 		int is_text, is_float, maybe_float, has_digits;
-		double d;
+		int seen_exp;
 		char *p;
 
+		lconv = localeconv ();
+		
 		is_text = is_float = maybe_float = has_digits = FALSE;
+		seen_exp = FALSE;
 		for (p = text; *p && !is_text; p++){
 			switch (*p){
 			case '0': case '1': case '2': case '3': case '4':
@@ -291,11 +516,22 @@ cell_set_text_simple (Cell *cell, char *text)
 			case '-':
 				if (p == text)
 					break;
+				if (seen_exp)
+					is_text = TRUE;
 				/* falldown */
 				
-			case 'E': case 'e': case '+': case ':': case '.':
+			case 'E': case 'e': case '+': case ':': case '.': case ',':
+				if (*p == 'e' || *p == 'E')
+					seen_exp = TRUE;
+				
+				if (*p == ',' || *p == '.')
+					if (*lconv->decimal_point != *p){
+						is_text = TRUE;
+						break;
+					}
 				maybe_float = TRUE;
 				break;
+
 			default:
 				is_text = TRUE;
 			}
@@ -354,22 +590,69 @@ cell_content_changed (Cell *cell)
 void
 cell_set_text (Cell *cell, char *text)
 {
-	GList   *deps;
-
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (text != NULL);
 	
 	cell_queue_redraw (cell);
 
 	cell_set_text_simple (cell, text);
-
 	cell_content_changed (cell);
 
 	cell_queue_redraw (cell);
 }
 
-/*
- * Makes a copy of a Cell
+/**
+ * cell_set_formula_tree_simple:
+ * @cell:    the cell to set the formula to
+ * @formula: an expression tree with the formula
+ *
+ * This is an internal function.  It should be only called by routines that
+ * know what they are doing.  These are the important differences from
+ * cell_set_formula:
+ *
+ *   - It does not queue redraws (so you have to queue the redraw yourself
+ *     or queue a full redraw).
+ *
+ *   - It does not queue any recomputations.  YOu have to queue the
+ *     recompute yourself. 
+ */
+void
+cell_set_formula_tree_simple (Cell *cell, ExprTree *formula)
+{
+	g_return_if_fail (cell != NULL);
+	g_return_if_fail (formula != NULL);
+
+	cell_modified (cell);
+
+	if (cell->parsed_node)
+		expr_tree_unref (cell->parsed_node);
+
+	cell->parsed_node = formula;
+	expr_tree_ref (formula);
+	cell_formula_changed (cell);
+}
+
+void
+cell_set_formula_tree (Cell *cell, ExprTree *formula)
+{
+	g_return_if_fail (cell != NULL);
+
+	cell_queue_redraw (cell);
+
+	cell_set_formula_tree_simple (cell, formula);
+	cell_content_changed (cell);
+
+	cell_queue_redraw (cell);
+}
+
+/**
+ * cell_copy:
+ * @cell: existing cell to duplicate
+ *
+ * Makes a copy of a Cell.  
+ *
+ * Returns a copy of the cell.  Note that the col, row and sheet
+ * fields are set to NULL.
  */
 Cell *
 cell_copy (Cell *cell)
@@ -383,8 +666,11 @@ cell_copy (Cell *cell)
 	/* bitmap copy first */
 	*new_cell = *cell;
 
+	new_cell->col   = NULL;
+	new_cell->row   = NULL;
+	new_cell->sheet = NULL;
+	
 	/* now copy propertly the rest */
-	string_ref      (new_cell->entered_text);
 	if (new_cell->parsed_node)
 		expr_tree_ref   (new_cell->parsed_node);
 	string_ref      (new_cell->text);
@@ -392,6 +678,10 @@ cell_copy (Cell *cell)
 	new_cell->style = style_duplicate (new_cell->style);
 	new_cell->value = value_duplicate (new_cell->value);
 
+	new_cell->comment = NULL;
+	if (cell->comment)
+		cell_set_comment (new_cell, cell->comment->comment->str);
+
 	return new_cell;
 }
 
@@ -400,6 +690,8 @@ cell_destroy (Cell *cell)
 {
 	g_return_if_fail (cell != NULL);
 
+	cell_modified (cell);
+
 	if (cell->parsed_node){
 		expr_tree_unref (cell->parsed_node);
 	}
@@ -407,13 +699,15 @@ cell_destroy (Cell *cell)
 	if (cell->render_color)
 		style_color_unref (cell->render_color);
 
-	if (cell->comment)
-		string_unref (cell->comment);
-	
-	string_unref  (cell->entered_text);
-	string_unref  (cell->text);
+	cell_comment_destroy (cell);
+
+	if (cell->text)
+		string_unref  (cell->text);
+
 	style_destroy (cell->style);
 	value_release (cell->value);
+
+	g_free (cell);
 }
 
 void
@@ -486,8 +780,10 @@ cell_set_format_simple (Cell *cell, char *format)
 		return;
 
 	/* Change the format */
+	cell_modified (cell);
 	style_format_unref (cell->style->format);
 	cell->style->format = style_format_new (format);
+	cell->flags |= CELL_FORMAT_SET;
 }
 
 /*
@@ -506,69 +802,63 @@ cell_set_format (Cell *cell, char *format)
 	if (strcmp (format, cell->style->format->format) == 0)
 		return;
 
+	cell_modified (cell);
 	cell_queue_redraw (cell);
 	
 	/* Change the format */
 	style_format_unref (cell->style->format);
 	cell->style->format = style_format_new (format);
-
+	cell->flags |= CELL_FORMAT_SET;
+	
 	/* re-render the cell text */
 	cell_render_value (cell);
 	cell_queue_redraw (cell);
 }
 
 void
-cell_set_comment (Cell *cell, char *str)
+cell_comment_reposition (Cell *cell)
 {
-	int had_comments = FALSE;
+	GList *l;
 	
 	g_return_if_fail (cell != NULL);
-	g_return_if_fail (str != NULL);
-
-	if (cell->comment){
-		string_unref (cell->comment);
-		had_comments = TRUE;
-	}
+	g_return_if_fail (cell->comment != NULL);
 	
-	cell->comment = string_get (str);
+	for (l = cell->comment->realized_list; l; l = l->next){
+		GnomeCanvasItem *o = l->data;
+		SheetView *sheet_view = GNUMERIC_SHEET (o->canvas)->sheet_view;
 
-	if (had_comments)
-		cell_queue_redraw (cell);
+		sheet_view_comment_relocate (sheet_view, cell->col->pos, cell->row->pos, o);
+	}
 }
 
 /*
- * cell_formula_relocate:
+ * cell_relocate:
  * @cell:       The cell that is changing position
  * @target_col: The new column
  * @target_row: The new row.
  *
  * This routine is used to move a cell to a different location:
- * The parsed tree is decoded as if it were evaluated at the new position
- * and then it is reparsed (important only for keeping the ->entered_text
- * information syncronized).
+ *
+ * Auxiliary items canvas items attached to the cell are moved.
  */
 void
-cell_formula_relocate (Cell *cell, int target_col, int target_row)
+cell_relocate (Cell *cell, int target_col, int target_row)
 {
-	char *text, *formula;
-	
 	g_return_if_fail (cell != NULL);
-	g_return_if_fail (cell->entered_text);
-	g_return_if_fail (cell->parsed_node);
-	
-	string_unref (cell->entered_text);
-	
-	text = expr_decode_tree (cell->parsed_node, target_col, target_row);
-	
-	formula = g_copy_strings ("=", text, NULL);
-	cell->entered_text = string_get (formula);
 
-	cell_set_formula (cell, formula);
-	
-	g_free (formula);
-	g_free (text);
-	
-	cell_formula_changed (cell);
+	/* 1. Tag the cell as modified */
+	cell_modified (cell);
+
+	/* 2. If the cell contains a formula, relocate the formula */
+	if (cell->parsed_node){
+		char *text, *formula;
+
+		expr_tree_ref (cell->parsed_node);
+		cell_formula_changed (cell);
+	}
+
+	/* 3. Move any auxiliary canvas items */
+	cell_comment_reposition (cell);
 }
 
 /*
@@ -580,10 +870,7 @@ cell_make_value (Cell *cell)
 	g_return_if_fail (cell != NULL);
 	g_return_if_fail (cell->parsed_node != NULL);
 
-	expr_tree_unref (cell->parsed_node);
-	cell->parsed_node = NULL;
-	string_unref (cell->entered_text);
-	cell->entered_text = string_ref (cell->text);
+	cell_modified (cell);
 }
 
 int
@@ -858,8 +1145,8 @@ calc_text_dimensions (int is_number, Style *style, char *text, int cell_w, int c
 void
 cell_calc_dimensions (Cell *cell)
 {
-	char    *rendered_text;
-	int     left, right;
+	char *rendered_text;
+	int  left, right;
 
 	g_return_if_fail (cell != NULL);
 
@@ -869,14 +1156,13 @@ cell_calc_dimensions (Cell *cell)
 		Style *style = cell->style;
 		int h, w;
 		
-		rendered_text = CELL_TEXT_GET (cell);
-
+		rendered_text = cell->text->str;
 		calc_text_dimensions (cell_is_number (cell),
 				      style, rendered_text,
 				      COL_INTERNAL_WIDTH (cell->col),
 				      ROW_INTERNAL_HEIGHT (cell->row),
 				      &h, &w);
-
+		
 		cell->width = cell->col->margin_a + cell->col->margin_b + w;
 		cell->height = cell->row->margin_a + cell->row->margin_b + h;
 
@@ -961,7 +1247,7 @@ cell_split_text (GdkFont *font, char *text, int width)
 	}
 	if (*line_begin){
 		line_len = p - line_begin;
-		line = g_malloc (line_len);
+		line = g_malloc (line_len+1);
 		memcpy (line, line_begin, line_len);
 		line [line_len] = 0;
 		list = g_list_append (list, line);
@@ -992,36 +1278,13 @@ str_trim_spaces (char *s)
 	while (p >= s){
 		if (*p == ' ')
 			*p = 0;
+		else
+			break;
 		p--;
 	}
 	return s;
 }
 
-#define TRIANGLE_WIDTH 6
-
-static void
-cell_draw_comment (Cell *cell, GdkGC *gc, GdkDrawable *drawable, int x1, int y1)
-{
-	GdkPoint points [3];
-	int width;
-	
-	gdk_gc_set_foreground (gc, &gs_red);
-	gdk_gc_set_background (gc, &gs_red);
-
-	width = cell->col->pixels;
-	y1++;
-	points [0].x = x1 + width;
-	points [0].y = y1;
-	points [1].x = x1 + width;
-	points [1].y = y1 + TRIANGLE_WIDTH;
-	points [2].x = x1 + width - TRIANGLE_WIDTH;
-	points [2].y = y1;
-	gdk_draw_polygon (drawable, gc, TRUE, points, 3);
-
-	gdk_gc_set_background (gc, &gs_white);
-	gdk_gc_set_foreground (gc, &gs_black);
-}
-
 /*
  * Returns the number of columns used for the draw
  */
@@ -1050,7 +1313,6 @@ cell_draw (Cell *cell, void *sv, GdkGC *gc, GdkDrawable *drawable, int x1, int y
 	font_height = font->ascent + font->descent;
 	
 	halign = cell_get_horizontal_align (cell);
-	text = CELL_TEXT_GET (cell);
 
 	/* if a number overflows, do special drawing */
 	if (width < cell->width && cell_is_number (cell)){
@@ -1064,6 +1326,8 @@ cell_draw (Cell *cell, void *sv, GdkGC *gc, GdkDrawable *drawable, int x1, int y
 	else
 		do_multi_line = FALSE;
 
+	text = cell->text->str;
+	
 	if (do_multi_line){
 		GList *lines, *l;
 		int cell_pixel_height = ROW_INTERNAL_HEIGHT (cell->row);
@@ -1140,7 +1404,7 @@ cell_draw (Cell *cell, void *sv, GdkGC *gc, GdkDrawable *drawable, int x1, int y
 			gdk_draw_text (drawable, font, gc, x1 + x_offset, y1 + y_offset, str, strlen (str));
 			y_offset += inter_space;
 			
-			g_free (str);
+			g_free (l->data);
 		}
 		g_list_free (lines);
 		
@@ -1201,10 +1465,34 @@ cell_draw (Cell *cell, void *sv, GdkGC *gc, GdkDrawable *drawable, int x1, int y
 		} while (halign == HALIGN_FILL && total < rect.width);
 	}
 
-	/* Draw the comment indicator */
-	if (cell->comment)
-		cell_draw_comment (cell, gc, drawable, x1, y1);
-
 	return end_col - start_col + 1;
 }
 
+char *
+cell_get_text (Cell *cell)
+{
+	char *str;
+	
+	g_return_val_if_fail (cell != NULL, NULL);
+
+	if (cell->parsed_node){
+		char *func, *ret;
+		
+		func = expr_decode_tree (cell->parsed_node, cell->sheet, cell->col->pos, cell->row->pos);
+		ret = g_copy_strings ("=", func, NULL);
+		g_free (func);
+
+		return ret;
+	}
+
+	/*
+	 * If a value is set, return that text formatted
+	 */
+	if (cell->value)
+		str = format_value (cell->style->format, cell->value, NULL);
+	else
+		str = g_strdup (cell->text->str);
+	
+	return str;
+}
+
