@@ -12,7 +12,7 @@
 #include "dialogs.h"
 
 void
-dialog_insert_cells (Sheet *sheet)
+dialog_insert_cells (Workbook *wb, Sheet *sheet)
 {
 	int state [4] = { 1, 0, 0, 0 };
 	SheetSelection *ss;
