@@ -1,5 +1,5 @@
-#ifndef EXPR_H
-#define EXPR_H
+#ifndef GNUMERIC_EXPR_H
+#define GNUMERIC_EXPR_H
 
 typedef enum {
 	OPER_EQUAL,
@@ -20,7 +20,7 @@ typedef enum {
 
         OPER_CONSTANT,
 	OPER_VAR,
-	OPER_NEG
+	OPER_NEG,
 } Operation;
 
 typedef enum {
@@ -31,17 +31,9 @@ typedef enum {
 	VALUE_ARRAY,
 } ValueType;
 
-/*
- * We use the GNU Multi-precission library for storing our 
- * numbers
- */
 typedef struct {
-	int col;
-	int row;
-} CellPos;
-
-typedef struct {
-	int col, row;
+	void *sheet;
+	int   col, row;
 
 	unsigned int col_relative:1;
 	unsigned int row_relative:1;
@@ -134,14 +126,18 @@ extern char     *parser_expr;
 extern char     *parser_desired_format;
 extern ParseErr  parser_error;
 extern ExprTree *parser_result;
+extern void     *parser_sheet;
 extern int       parser_col, parser_row;
 
-void        cell_get_abs_col_row (CellRef *cell_ref, int eval_col, int eval_row, int *col, int *row);
+void        cell_get_abs_col_row (CellRef *cell_ref,
+				  int eval_col, int eval_row,
+				  int *col, int *row);
 
-ExprTree   *expr_parse_string    (char *expr, int col, int row,
+ExprTree   *expr_parse_string    (char *expr, void *sheet, int col, int row,
 				  char **desired_format, char **error_msg);
 
-char       *expr_decode_tree     (ExprTree *tree, int col, int row);
+char       *expr_decode_tree     (ExprTree *tree, void *sheet,
+				  int col, int row);
 
 void        expr_tree_ref        (ExprTree *tree);
 void        expr_tree_unref      (ExprTree *tree);
@@ -154,6 +150,7 @@ void        value_release        (Value *value);
 Value      *value_cast_to_float  (Value *v);
 int         value_get_bool       (Value *v, int *err);
 float_t     value_get_as_double  (Value *v);
+int         value_get_as_int     (Value *v);
 void        value_copy_to        (Value *dest, Value *source);
 				 
 void        value_dump           (Value *value);
@@ -169,4 +166,4 @@ int         yyparse              (void);
 void        functions_init       (void);
 void        constants_init       (void);
 
-#endif
+#endif /* GNUMERIC_EXPR_H */
