@@ -25,8 +25,6 @@ GdkColor gs_white, gs_black, gs_light_gray, gs_dark_gray, gs_red;
 
 
 static GnomeCanvasClass *sheet_parent_class;
-static void stop_cell_selection (GnumericSheet *gsheet);
-
 
 static void
 gnumeric_sheet_destroy (GtkObject *object)
@@ -114,7 +112,7 @@ gnumeric_sheet_cursor_set (GnumericSheet *gsheet, int col, int row)
  *
  * Set the current selection to cover the inclusive area delimited by
  * start_col, start_row, end_col and end_row.  The actual cursor is
- * placed at start_col, start_row
+ * placed at base_col, base_row
  */
 void
 gnumeric_sheet_set_selection (GnumericSheet *gsheet, SheetSelection *ss)
@@ -125,6 +123,7 @@ gnumeric_sheet_set_selection (GnumericSheet *gsheet, SheetSelection *ss)
 
 	sheet_cursor_set (
 		gsheet->sheet_view->sheet,
+		ss->base_col, ss->base_row,
 		ss->start_col, ss->start_row,
 		ss->end_col, ss->end_row);
 }
@@ -147,7 +146,8 @@ move_cursor (GnumericSheet *gsheet, int col, int row, gboolean clear_selection)
 	if (clear_selection)
 		sheet_selection_reset_only (sheet);
 
-	sheet_cursor_set (sheet, col, row, col, row);
+	sheet_make_cell_visible (sheet, col, row);
+	sheet_cursor_set (sheet, col, row, col, row, col, row);
 
 	if (clear_selection)
 		sheet_selection_append (sheet, col, row);
@@ -242,7 +242,7 @@ move_vertical_selection (GnumericSheet *gsheet, int count)
  * a cell range (if the cursor is in a spot in the expression
  * where it makes sense to have a cell reference), false if not.
  */
-static int
+int
 gnumeric_sheet_can_move_cursor (GnumericSheet *gsheet)
 {
 	GtkEntry *entry = GTK_ENTRY (gsheet->entry);
@@ -267,11 +267,10 @@ gnumeric_sheet_can_move_cursor (GnumericSheet *gsheet)
 }
 
 static void
-start_cell_selection (GnumericSheet *gsheet)
+start_cell_selection_at (GnumericSheet *gsheet, int col, int row)
 {
 	GnomeCanvas *canvas = GNOME_CANVAS (gsheet);
 	GnomeCanvasGroup *group = GNOME_CANVAS_GROUP (canvas->root);
-	Sheet *sheet = gsheet->sheet_view->sheet;
 	
 	g_return_if_fail (gsheet->selecting_cell == FALSE);
 	
@@ -282,19 +281,40 @@ start_cell_selection (GnumericSheet *gsheet)
 		"Sheet", gsheet->sheet_view->sheet,
 		"Grid",  gsheet->item_grid,
 		"Style", ITEM_CURSOR_ANTED, NULL));
-	item_cursor_set_bounds (ITEM_CURSOR (gsheet->selection),
-				sheet->cursor_col,
-				sheet->cursor_row,
-				sheet->cursor_col,
-				sheet->cursor_row);
+	gsheet->selection->base_col = col;
+	gsheet->selection->base_row = row;
+	item_cursor_set_bounds (ITEM_CURSOR (gsheet->selection), col, row, col, row);
 				
 	gsheet->sel_cursor_pos = GTK_EDITABLE (gsheet->entry)->current_pos;
 	gsheet->sel_text_len = 0;
 }
 
 static void
-stop_cell_selection (GnumericSheet *gsheet)
+start_cell_selection (GnumericSheet *gsheet)
 {
+	Sheet *sheet = gsheet->sheet_view->sheet;
+
+	start_cell_selection_at (gsheet, sheet->cursor_col, sheet->cursor_row);
+}
+
+void
+gnumeric_sheet_start_cell_selection (GnumericSheet *gsheet, int col, int row)
+{
+	g_return_if_fail (gsheet != NULL);
+	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
+
+	if (gsheet->selecting_cell)
+		return;
+
+	start_cell_selection_at (gsheet, col, row);
+}
+
+void
+gnumeric_sheet_stop_cell_selection (GnumericSheet *gsheet)
+{
+	g_return_if_fail (gsheet != NULL);
+	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
+
 	if (!gsheet->selecting_cell)
 		return;
 	
@@ -342,7 +362,7 @@ gnumeric_sheet_destroy_editing_cursor (GnumericSheet *gsheet)
 	g_return_if_fail (gsheet != NULL);
 	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
 	
-	stop_cell_selection (gsheet);
+	gnumeric_sheet_stop_cell_selection (gsheet);
 	
 	if (!gsheet->item_editor)
 		return;
@@ -396,6 +416,53 @@ selection_insert_selection_string (GnumericSheet *gsheet)
 				   gsheet->sel_text_len);
 }
 
+/*
+ * Invoked by Item-Grid to extend the cursor selection
+ */
+void
+gnumeric_sheet_selection_extend (GnumericSheet *gsheet, int col, int row)
+{
+	ItemCursor *ic;
+	
+	g_return_if_fail (gsheet != NULL);
+	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
+	g_return_if_fail (gsheet->selecting_cell);
+	g_return_if_fail (col < SHEET_MAX_COLS);
+	g_return_if_fail (row < SHEET_MAX_ROWS);
+
+	ic = gsheet->selection;
+
+	selection_remove_selection_string (gsheet);
+	item_cursor_set_bounds (ic,
+				MIN (ic->base_col, col),
+				MIN (ic->base_row, row),
+				MAX (ic->base_col, col),
+				MAX (ic->base_row, row));
+	selection_insert_selection_string (gsheet);
+}
+
+/*
+ * Invoked by Item-Grid to place the selection cursor on a specific
+ * spot.
+ */
+void
+gnumeric_sheet_selection_cursor_place (GnumericSheet *gsheet, int col, int row)
+{
+	ItemCursor *ic;
+	
+	g_return_if_fail (gsheet != NULL);
+	g_return_if_fail (GNUMERIC_IS_SHEET (gsheet));
+	g_return_if_fail (gsheet->selecting_cell);
+	g_return_if_fail (col < SHEET_MAX_COLS);
+	g_return_if_fail (row < SHEET_MAX_ROWS);
+	
+	ic = gsheet->selection;
+	
+	selection_remove_selection_string (gsheet);
+	item_cursor_set_bounds (ic, col, row, col, row);
+	selection_insert_selection_string (gsheet);
+}
+
 static void
 selection_cursor_move_horizontal (GnumericSheet *gsheet, int dir)
 {
@@ -467,19 +534,17 @@ selection_expand_horizontal (GnumericSheet *gsheet, int dir)
 	}
 
 	ic = gsheet->selection;
-	if (dir == -1){
-		if (ic->start_col == 0)
+	if (ic->end_col == SHEET_MAX_COLS-1)
 			return;
-	} else {
-		if (ic->end_col == SHEET_MAX_COLS-1)
-			return;
-	}
 
+	if (dir == -1 && ic->start_col == ic->end_col)
+		return;
+	
 	selection_remove_selection_string (gsheet);
 	item_cursor_set_bounds (ic,
-				ic->start_col + (dir == -1 ? dir : 0),
+				ic->start_col,
 				ic->start_row,
-				ic->end_col  +  (dir == 1 ? dir : 0),
+				ic->end_col  + dir,
 				ic->end_row);
 	selection_insert_selection_string (gsheet);
 }
@@ -498,20 +563,18 @@ selection_expand_vertical (GnumericSheet *gsheet, int dir)
 
 	ic = gsheet->selection;
 	
-	if (dir == -1){
-		if (ic->start_row == 0)
-			return;
-	} else {
-		if (ic->end_row == SHEET_MAX_ROWS-1)
-			return;
-	}
+	if (ic->end_row == SHEET_MAX_ROWS-1)
+		return;
 
+	if (dir == -1 && ic->start_row == ic->end_row)
+		return;
+	
 	selection_remove_selection_string (gsheet);
 	item_cursor_set_bounds (ic,
 				ic->start_col,
-				ic->start_row + (dir == -1 ? dir : 0),
+				ic->start_row,
 				ic->end_col,
-				ic->end_row + (dir == 1 ? dir : 0));	
+				ic->end_row + dir);
 	selection_insert_selection_string (gsheet);
 }
 
@@ -526,11 +589,11 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	void (*movefn_horizontal) (GnumericSheet *, int);
 	void (*movefn_vertical)   (GnumericSheet *, int);
 	int  cursor_move = gnumeric_sheet_can_move_cursor (gsheet);
-
+	
 	if ((event->state & GDK_SHIFT_MASK) != 0){
 		if (cursor_move){
 			movefn_horizontal = selection_expand_horizontal;
-			movefn_vertical   = selection_expand_vertical;
+			movefn_vertical = selection_expand_vertical;
 		} else {
 			movefn_horizontal = move_horizontal_selection;
 			movefn_vertical   = move_vertical_selection;
@@ -550,9 +613,9 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	 */
 	if (cursor_move){
 		switch (event->keyval){
-		case GDK_Shift_L: case GDK_Shift_R:
-		case GDK_Alt_L:   case GDK_Alt_R:
-		case GDK_Control_L:   case GDK_Control_R:
+		case GDK_Shift_L:   case GDK_Shift_R:
+		case GDK_Alt_L:     case GDK_Alt_R:
+		case GDK_Control_L: case GDK_Control_R:
 			return 1;
 		}
 	}
@@ -598,32 +661,40 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 	}
 
 	switch (event->keyval){
+	case GDK_KP_Left:
 	case GDK_Left:
 		(*movefn_horizontal)(gsheet, -1);
 		break;
 
+	case GDK_KP_Right:
 	case GDK_Right:
 		(*movefn_horizontal)(gsheet, 1);
 		break;
 
+	case GDK_KP_Up:
 	case GDK_Up:
 		(*movefn_vertical)(gsheet, -1);
 		break;
 
+	case GDK_KP_Down:
 	case GDK_Down:
 		(*movefn_vertical)(gsheet, 1);
 		break;
 
+	case GDK_KP_Page_Up:
 	case GDK_Page_Up:
 	        (*movefn_vertical)(gsheet, -(gsheet->last_visible_row-gsheet->top_row));
 		break;
 
+	case GDK_KP_Page_Down:
 	case GDK_Page_Down:
 	        (*movefn_vertical)(gsheet, gsheet->last_visible_row-gsheet->top_row);
 		break;
 
+	case GDK_KP_Home:
 	case GDK_Home:
 		if ((event->state & GDK_CONTROL_MASK) != 0){
+			sheet_make_cell_visible (sheet, 0, 0);
 			sheet_cursor_move (sheet, 0, 0);
 			break;
 		} else
@@ -635,11 +706,13 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 		sheet_selection_clear (sheet);
 		break;
 
+	case GDK_KP_Enter:
 	case GDK_Return:
 		if ((event->state & GDK_CONTROL_MASK) != 0){
 			if (gsheet->item_editor){
 				Cell *cell;
-
+				char *text;
+				
 				sheet_accept_pending_input (sheet);
 				cell = sheet_cell_get (sheet,
 						       sheet->cursor_col,
@@ -652,9 +725,9 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 				 * part of the code
 				 */
 				g_return_val_if_fail (cell != NULL, 1);
-				g_return_val_if_fail (cell->entered_text != NULL, 1);
-				sheet_fill_selection_with (
-					sheet, cell->entered_text->str);
+				text = cell_get_text (cell);
+				sheet_fill_selection_with (sheet, text);
+				g_free (text);
 			}
 			return 1;
 		}
@@ -691,10 +764,11 @@ gnumeric_sheet_key_mode_sheet (GnumericSheet *gsheet, GdkEventKey *event)
 			if ((event->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) != 0)
 				return 0;
 			
-			if (event->keyval >= 0x20 && event->keyval <= 0xff)
-			    sheet_start_editing_at_cursor (sheet);
+			if ((event->keyval >= 0x20 && event->keyval <= 0xff) ||
+			    (event->keyval >= GDK_KP_Add && event->keyval <= GDK_KP_9))
+				sheet_start_editing_at_cursor (sheet);
 		}
-		stop_cell_selection (gsheet);
+		gnumeric_sheet_stop_cell_selection (gsheet);
 		
 		/* Forward the keystroke to the input line */
 		gtk_widget_event (gsheet->entry, (GdkEvent *) event);
@@ -767,7 +841,7 @@ gnumeric_sheet_new (SheetView *sheet_view, ItemBar *colbar, ItemBar *rowbar)
 	
 	entry = workbook->ea_input;
 	gsheet = gnumeric_sheet_create (sheet_view, entry);
-	gnome_canvas_set_size (GNOME_CANVAS (gsheet), 300, 100);
+	gtk_widget_set_usize (GTK_WIDGET (gsheet), 300, 100);
 	
 	/* FIXME: figure out some real size for the canvas scrolling region */
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (gsheet), 0, 0, 1000000, 1000000);
@@ -866,11 +940,13 @@ void
 gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet)
 {
 	GnomeCanvas   *canvas = GNOME_CANVAS (gsheet);
-	int pixels, col, row;
+	int pixels, col, row, width, height;
 
 	/* Find out the last visible col and the last full visible column */
 	pixels = 0;
 	col = gsheet->top_col;
+	width = GTK_WIDGET (canvas)->allocation.width;
+	
 	do {
 		ColRowInfo *ci;
 		int cb;
@@ -878,10 +954,10 @@ gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet)
 		ci = sheet_col_get_info (gsheet->sheet_view->sheet, col);
 		cb = pixels + ci->pixels;
 		
-		if (cb == canvas->width){
+		if (cb == width){
 			gsheet->last_visible_col = col;
 			gsheet->last_full_col = col;
-		} if (cb > canvas->width){
+		} if (cb > width){
 			gsheet->last_visible_col = col;
 			if (col == gsheet->top_col)
 				gsheet->last_full_col = gsheet->top_col;
@@ -890,11 +966,12 @@ gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet)
 		}
 		pixels = cb;
 		col++;
-	} while (pixels < canvas->width);
+	} while (pixels < width);
 
 	/* Find out the last visible row and the last fully visible row */
 	pixels = 0;
 	row = gsheet->top_row;
+	height = GTK_WIDGET (canvas)->allocation.height;
 	do {
 		ColRowInfo *ri;
 		int cb;
@@ -902,10 +979,10 @@ gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet)
 		ri = sheet_row_get_info (gsheet->sheet_view->sheet, row);
 		cb = pixels + ri->pixels;
 		
-		if (cb == canvas->height){
+		if (cb == height){
 			gsheet->last_visible_row = row;
 			gsheet->last_full_row = row;
-		} if (cb > canvas->height){
+		} if (cb > height){
 			gsheet->last_visible_row = row;
 			if (col == gsheet->top_row)
 				gsheet->last_full_row = gsheet->top_row;
@@ -914,17 +991,22 @@ gnumeric_sheet_compute_visible_ranges (GnumericSheet *gsheet)
 		}
 		pixels = cb;
 		row++;
-	} while (pixels < canvas->height);
+	} while (pixels < height);
 }
 
 static int
-gnumeric_sheet_set_top_row (GnumericSheet *gsheet, int new_top_row)
+gnumeric_sheet_bar_set_top_row (GnumericSheet *gsheet, int new_top_row)
 {
-	GnomeCanvas *rowc = GNOME_CANVAS_ITEM (gsheet->rowbar)->canvas;
-	Sheet *sheet = gsheet->sheet_view->sheet;
+	GnomeCanvas *rowc;
+	Sheet *sheet;
 	int row_distance;
 	int x;
-	
+
+	g_return_val_if_fail (gsheet != NULL, 0);
+	g_return_val_if_fail (new_top_row >= 0 && new_top_row <= SHEET_MAX_ROWS-1, 0);
+
+	rowc = GNOME_CANVAS_ITEM (gsheet->rowbar)->canvas;
+	sheet = gsheet->sheet_view->sheet;
 	gsheet->top_row = new_top_row;
 	row_distance = sheet_row_get_distance (sheet, 0, gsheet->top_row);
 
@@ -934,14 +1016,33 @@ gnumeric_sheet_set_top_row (GnumericSheet *gsheet, int new_top_row)
 	return row_distance;
 }
 
-static int
-gnumeric_sheet_set_top_col (GnumericSheet *gsheet, int new_top_col)
+void
+gnumeric_sheet_set_top_row (GnumericSheet *gsheet, int new_top_row)
 {
-	GnomeCanvas *colc = GNOME_CANVAS_ITEM (gsheet->colbar)->canvas;
-	Sheet *sheet = gsheet->sheet_view->sheet;
+	int distance, x;
+	
+	g_return_if_fail (gsheet != NULL);
+	g_return_if_fail (new_top_row >= 0 && new_top_row <= SHEET_MAX_ROWS-1);
+
+	distance = gnumeric_sheet_bar_set_top_row (gsheet, new_top_row);
+	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (gsheet), &x, NULL);
+	gnome_canvas_scroll_to (GNOME_CANVAS (gsheet), x, distance);
+}
+
+static int
+gnumeric_sheet_bar_set_top_col (GnumericSheet *gsheet, int new_top_col)
+{
+	GnomeCanvas *colc;
+	Sheet *sheet;
 	int col_distance;
 	int y;
 
+	g_return_val_if_fail (gsheet != NULL, 0);
+	g_return_val_if_fail (new_top_col >= 0 && new_top_col <= SHEET_MAX_COLS-1, 0);
+
+	colc = GNOME_CANVAS_ITEM (gsheet->colbar)->canvas;
+	sheet = gsheet->sheet_view->sheet;
+
 	gsheet->top_col = new_top_col;
 	col_distance = sheet_col_get_distance (sheet, 0, gsheet->top_col);
 
@@ -951,6 +1052,19 @@ gnumeric_sheet_set_top_col (GnumericSheet *gsheet, int new_top_col)
 	return col_distance;
 }
 
+void
+gnumeric_sheet_set_top_col (GnumericSheet *gsheet, int new_top_col)
+{
+	int distance, y;
+	
+	g_return_if_fail (gsheet != NULL);
+	g_return_if_fail (new_top_col >= 0 && new_top_col <= SHEET_MAX_COLS-1);
+
+	distance = gnumeric_sheet_bar_set_top_col (gsheet, new_top_col);
+	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (gsheet), NULL, &y);
+	gnome_canvas_scroll_to (GNOME_CANVAS (gsheet), distance, y);
+}
+
 void
 gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row)
 {
@@ -975,7 +1089,7 @@ gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row)
 		new_top_col = col;
 	} else if (col > gsheet->last_full_col){
 		ColRowInfo *ci;
-		int width = canvas->width;
+		int width = GTK_WIDGET (canvas)->allocation.width;
 		int allocated = 0;
 		int first_col;
 
@@ -995,7 +1109,7 @@ gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row)
 		new_top_row = row;
 	} else if (row > gsheet->last_full_row){
 		ColRowInfo *ri;
-		int height = canvas->height;
+		int height = GTK_WIDGET (canvas)->allocation.height;
 		int allocated = 0;
 		int first_row;
 
@@ -1015,12 +1129,12 @@ gnumeric_sheet_make_cell_visible (GnumericSheet *gsheet, int col, int row)
 	gnome_canvas_get_scroll_offsets (GNOME_CANVAS (gsheet), &col_distance, &row_distance);
 	
 	if (gsheet->top_col != new_top_col){
-		col_distance = gnumeric_sheet_set_top_col (gsheet, new_top_col);
+		col_distance = gnumeric_sheet_bar_set_top_col (gsheet, new_top_col);
 		did_change = 1;
 	}
 
 	if (gsheet->top_row != new_top_row){
-		row_distance = gnumeric_sheet_set_top_row (gsheet, new_top_row);
+		row_distance = gnumeric_sheet_bar_set_top_row (gsheet, new_top_row);
 		did_change = 1;
 	}
 
