@@ -18,7 +18,6 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
-#include <libgnome/lib_date.h>
 #include <regex.h>
 #include "number-match.h"
 #include "formats.h"
@@ -49,7 +48,7 @@ create_option_list (char **list)
 	res [0] = '(';
 	res [1] = 0;
 	for (p = list; *p; p++){
-		char *v = *p;
+		char *v = _(*p);
 
 		if (*v == '*')
 			v++;
@@ -88,8 +87,6 @@ format_create_regexp (char *format, GByteArray **dest)
 	GString *regexp;
 	GByteArray *match_types;
 	char *str;
-	int top = 0;
-	int type;
 	int hour_seen = FALSE;
 	
 	g_return_val_if_fail (format != NULL, NULL);
@@ -172,13 +169,21 @@ format_create_regexp (char *format, GByteArray **dest)
 				if (*(format+1) == 'm'){
 					if (*(format+2) == 'm'){
 						if (*(format+3) == 'm'){
-							g_string_append (regexp,
-									 create_option_list (month_long));
+							char *l;
+
+							l = create_option_list (month_long);
+							g_string_append (regexp, l);
+							g_free (l);
+
 							append_type (MATCH_MONTH_FULL);
 							format++;
 						} else {
-							g_string_append (regexp,
-									 create_option_list (month_short));
+							char *l;
+
+							l = create_option_list (month_short);
+							g_string_append (regexp, l);
+							g_free (l);
+							
 							append_type (MATCH_MONTH_SHORT);
 						}
 						format++;
@@ -205,11 +210,20 @@ format_create_regexp (char *format, GByteArray **dest)
 			if (*(format+1) == 'd'){
 				if (*(format+2) == 'd'){
 					if (*(format+3) == 'd'){
-						g_string_append (regexp, create_option_list (day_long));
+						char *l;
+
+						l = create_option_list (day_long);
+						g_string_append (regexp, l);
+						g_free (l);
+						
 						append_type (MATCH_DAY_FULL);
 						format++;
 					} else {
-						g_string_append (regexp, create_option_list (day_short));
+						char *l;
+
+						l = create_option_list (day_short);
+						g_string_append (regexp, l);
+						g_free (l);
 					}
 					format++;
 				} else {
@@ -627,6 +641,7 @@ compute_value (char *s, regmatch_t *mp, GByteArray *array, double *v)
 	{
 		time_t t = time (NULL);
 		struct tm *tm = localtime (&t);
+		GDate *date;
 
 		if (!(year == -1 && month == -1 && day == -1)){
 			if (year == -1){
@@ -649,9 +664,11 @@ compute_value (char *s, regmatch_t *mp, GByteArray *array, double *v)
 			if (month < 1 || month > 12)
 				return FALSE;
 			
-			number =
-				calc_days (year, month, day) -
-				calc_days (1900, 1, 1) + 1;
+			date = g_date_new_dmy (day, month, year);
+
+			number = g_date_serial (date);
+
+			g_date_free (date);
 		}
 
 		*v = number;
@@ -697,7 +714,7 @@ format_match (char *s, double *v, char **format)
 
 	for (l = format_match_list; l; l = l->next){
 		format_parse_t *fp = l->data;
-		int i, b;
+		int b;
 		
 		if (regexec (&fp->regexp, s, NM, mp, 0) == REG_NOMATCH)
 			continue;
