@@ -13,9 +13,11 @@
 
 typedef struct 
 {
+	Workbook  *workbook;
 	GtkWidget *dialog;
 	GtkWidget *hbox;
 	GtkWidget *vbbox;
+	GtkWidget *scrollwin;
 	GtkWidget *clist;
 	GtkWidget *button_add;
 	GtkWidget *button_remove;
@@ -54,7 +56,7 @@ close_cb (GtkWidget *button, PluginManager *pm)
 static void
 add_cb (GtkWidget *button, PluginManager *pm)
 {
-	char *modfile = dialog_query_load_file ();
+	char *modfile = dialog_query_load_file (pm->workbook);
 	PluginData *pd;
 	
 	if (!modfile)
@@ -86,7 +88,7 @@ row_cb (GtkWidget * clist, gint row, gint col,
 }
 
 GtkWidget *
-plugin_manager_new (void)
+plugin_manager_new (Workbook *wb)
 {
 	PluginManager *pm;
 	gchar *n_titles[2] = { N_("Name"), N_("File") };
@@ -95,18 +97,26 @@ plugin_manager_new (void)
 	pm = g_new0 (PluginManager, 1);
 	if (!pm)
 		return NULL;
-	
+
+	pm->workbook = wb;
 	pm->dialog = gtk_window_new (GTK_WINDOW_DIALOG);
 	
 	pm->hbox = gtk_hbox_new (0, 0);
 	gtk_container_add (GTK_CONTAINER (pm->dialog), pm->hbox);
 
+	pm->scrollwin = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (pm->scrollwin),
+					GTK_POLICY_AUTOMATIC,
+					GTK_POLICY_AUTOMATIC); 
+	gtk_box_pack_start (GTK_BOX (pm->hbox), pm->scrollwin, 1, 1, 5);
+
 	titles [0] = _(n_titles [0]);
 	titles [1] = _(n_titles [1]);
 	
 	pm->clist = gtk_clist_new_with_titles (2, titles);
+	gtk_clist_column_titles_passive (GTK_CLIST (pm->clist));
 	gtk_widget_set_usize (pm->clist, 300, 120);
-	gtk_box_pack_start (GTK_BOX (pm->hbox), pm->clist, 1, 1, 5);
+	gtk_container_add (GTK_CONTAINER (pm->scrollwin), pm->clist);
 	
 	pm->vbbox = gtk_vbutton_box_new ();
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (pm->vbbox), GTK_BUTTONBOX_START);
