@@ -217,7 +217,7 @@ fill_item_new (Cell *cell)
 	value = cell->value;
 	value_type = value->type;
 	
-	if (CELL_IS_FORMULA (cell)){
+	if (cell->parsed_node){
 		fi->type = FILL_FORMULA;
 		fi->v.formula = cell->parsed_node;
 		
@@ -442,7 +442,7 @@ autofill_cell (Cell *cell, int idx, FillItem *fi)
 
 	case FILL_STRING_WITH_NUMBER: {
 		FillItem *last = fi->group_last;
-		char buffer [50], *v;
+		char buffer [sizeof (int) * 4], *v;
 		int i;
 		
 		i = last->v.numstr.num + idx * last->delta.d_int;
@@ -512,13 +512,7 @@ autofill_cell (Cell *cell, int idx, FillItem *fi)
 	}
 		
 	case FILL_FORMULA: {
-		char *text, *formula;
-		
-		text = expr_decode_tree (fi->v.formula, cell->col->pos, cell->row->pos);
-		formula = g_copy_strings ("=", text, NULL);
-		cell_set_text (cell, formula);
-		g_free (text);
-		g_free (formula);
+		cell_set_formula_tree (cell, fi->v.formula);
 		return;
 	}
 	
