@@ -100,22 +100,6 @@ sheet_view_redraw_cell_region (SheetView *sheet_view, int start_col, int start_r
 				     x+w, y+h);
 }
 
-void
-sheet_view_row_set_selection (SheetView *sheet_view, ColRowInfo *ri)
-{
-	gnome_canvas_request_redraw (
-		GNOME_CANVAS (sheet_view->row_canvas),
-		0, 0, INT_MAX, INT_MAX);
-}
-
-void
-sheet_view_col_set_selection (SheetView *sheet_view, ColRowInfo *ci)
-{
-	gnome_canvas_request_redraw (
-		GNOME_CANVAS (sheet_view->col_canvas),
-		0, 0, INT_MAX, INT_MAX);
-}
-
 void
 sheet_view_redraw_columns (SheetView *sheet_view)
 {
@@ -184,7 +168,7 @@ new_canvas_bar (SheetView *sheet_view, GtkOrientation o, GnomeCanvasItem **itemp
 		h = 20;
 		gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0, 0, 1000000, h);
 	}
-	gnome_canvas_set_size (GNOME_CANVAS (canvas), w, h);
+	gtk_widget_set_usize (canvas, w, h);
 	group = GNOME_CANVAS_GROUP (GNOME_CANVAS (canvas)->root);
 	item = gnome_canvas_item_new (group,
 				      item_bar_get_type (),
@@ -219,22 +203,34 @@ sheet_view_size_allocate (GtkWidget *widget, GtkAllocation *alloc, SheetView *sh
 }
 
 static void
-sheet_view_col_selection_changed (ItemBar *item_bar, int column, int reset, SheetView *sheet_view)
+sheet_view_col_selection_changed (ItemBar *item_bar, int column, int modifiers, SheetView *sheet_view)
 {	
-	ColRowInfo *ci;
 	Sheet *sheet = sheet_view->sheet;
 	
-	if (reset){
-		sheet_cursor_set (sheet, column, 0, column, SHEET_MAX_ROWS - 1);
-		sheet_selection_reset_only (sheet);
+	sheet_col_get (sheet, column);
+	
+	if (modifiers){
+		if ((modifiers & GDK_SHIFT_MASK) && sheet->selections){
+			SheetSelection *ss = sheet->selections->data;
+			int start_col, end_col;
+			
+			start_col = MIN (ss->base_col, column);
+			end_col = MAX (ss->base_col, column);
+			
+			sheet_selection_set (sheet,
+					     start_col, 0,
+					     end_col, SHEET_MAX_ROWS-1);
+			return;
+		}
+		sheet_cursor_move (sheet, column, 0);
+		if (!(modifiers & GDK_CONTROL_MASK))
+			sheet_selection_reset_only (sheet);
 		sheet_selection_append_range (sheet,
 					      column, 0,
 					      column, 0,
 					      column, SHEET_MAX_ROWS-1);
-		ci = sheet_col_get (sheet, column);
-		sheet_col_set_selection (sheet, ci, 1);
 	} else
-		sheet_selection_col_extend_to (sheet, column);
+		sheet_selection_extend_to (sheet, column, SHEET_MAX_ROWS - 1);
 }
 
 static void
@@ -242,37 +238,53 @@ sheet_view_col_size_changed (ItemBar *item_bar, int col, int width, SheetView *s
 {
 	Sheet *sheet = sheet_view->sheet;
 	GList *l;
+	ItemBarSelectionType type;
 
-	if (sheet_is_all_selected (sheet))
-		sheet_col_info_set_width (sheet, &sheet->default_col_style, width);
-		
-	for (l = sheet->cols_info; l; l = l->next){
-		ColRowInfo *ci = l->data;
+	type = sheet_col_selection_type (sheet, col);
 
-		if (ci->selected)
-			sheet_col_set_width (sheet, ci->pos, width);
-	}
-	sheet_col_set_width (sheet, col, width);
+ 	if (type == ITEM_BAR_FULL_SELECTION)
+ 		for (l = sheet->cols_info; l; l = l->next){
+ 			ColRowInfo *ci = l->data;
+ 
+ 			if (sheet_col_selection_type (sheet, ci->pos) == ITEM_BAR_FULL_SELECTION)
+ 				sheet_col_set_width (sheet, ci->pos, width);
+ 		}
+ 	else
+ 		sheet_col_set_width (sheet, col, width);
+	
 	gnumeric_sheet_compute_visible_ranges (GNUMERIC_SHEET (sheet_view->sheet_view));
 }
 
 static void
-sheet_view_row_selection_changed (ItemBar *item_bar, int row, int reset, SheetView *sheet_view)
+sheet_view_row_selection_changed (ItemBar *item_bar, int row, int modifiers, SheetView *sheet_view)
 {
-	ColRowInfo *ri;
 	Sheet *sheet = sheet_view->sheet;
+
+	sheet_row_get (sheet, row);
+	
+	if (modifiers){
+		if ((modifiers & GDK_SHIFT_MASK) && sheet->selections){
+			SheetSelection *ss = sheet->selections->data;
+			int start_row, end_row;
+			
+			start_row = MIN (ss->base_row, row);
+			end_row = MAX (ss->base_row, row);
+			
+			sheet_selection_set (sheet,
+					     0, start_row,
+					     SHEET_MAX_COLS-1, end_row);
+			return;
+		}
+		sheet_cursor_move (sheet, 0, row);
+		if (!(modifiers & GDK_CONTROL_MASK))
+ 			sheet_selection_reset_only (sheet);
 	
-	if (reset){
-		sheet_cursor_set (sheet, 0, row, SHEET_MAX_COLS-1, row);
-		sheet_selection_reset_only (sheet);
 		sheet_selection_append_range (sheet,
 					      0, row,
 					      0, row,
 					      SHEET_MAX_COLS-1, row);
-		ri = sheet_row_get (sheet, row);
-		sheet_row_set_selection (sheet, ri, 1);
 	} else
-		sheet_selection_row_extend_to (sheet, row);
+		sheet_selection_extend_to (sheet, SHEET_MAX_COLS-1, row);
 }
 
 static void
@@ -280,17 +292,21 @@ sheet_view_row_size_changed (ItemBar *item_bar, int row, int height, SheetView *
 {
 	Sheet *sheet = sheet_view->sheet;
 	GList *l;
-	
-	if (sheet_is_all_selected (sheet))
-		sheet_row_info_set_height (sheet, &sheet->default_row_style, height, TRUE);
-		
-	for (l = sheet->rows_info; l; l = l->next){
-		ColRowInfo *ri = l->data;
+	ItemBarSelectionType type;
+
+	type = sheet_row_selection_type (sheet, row);
+
+	if (type == ITEM_BAR_FULL_SELECTION)
+		for (l = sheet->rows_info; l; l = l->next){
+			ColRowInfo *ri = l->data;
+			
+			if (sheet_row_selection_type (sheet, ri->pos) == ITEM_BAR_FULL_SELECTION)
+					sheet_row_set_height (sheet, ri->pos, height, TRUE);
+			
+		}
+	else
+		sheet_row_set_height (sheet, row, height, TRUE);
 
-		if (ri->selected)
-			sheet_row_set_height (sheet, ri->pos, height, TRUE);
-	}
-	sheet_row_set_height (sheet, row, height, TRUE);
 	gnumeric_sheet_compute_visible_ranges (GNUMERIC_SHEET (sheet_view->sheet_view));
 }
 
@@ -303,7 +319,7 @@ button_select_all (GtkWidget *the_button, SheetView *sheet_view)
 static void
 set_tip_label (SheetView *sheet_view, char *format, GtkAdjustment *adj, int horizontal)
 {
-	char buffer [40];
+	char buffer [20 + sizeof (long) * 4];
 
 	if (horizontal)
 		snprintf (buffer, sizeof (buffer), format, col_name (adj->value));
@@ -366,13 +382,21 @@ horizontal_scroll_event (GtkScrollbar *scroll, GdkEvent *event, SheetView *sheet
 		set_tip_label (sheet_view, _("Column: %s"), GTK_ADJUSTMENT (sheet_view->ha), 1);
 		position_tooltip (sheet_view, 1);
 		gtk_widget_show_all (gtk_widget_get_toplevel (sheet_view->tip));
-	} else if (event->type == GDK_BUTTON_RELEASE){
+	}
+	else if (event->type == GDK_BUTTON_RELEASE)
+	{
+		GnumericSheet  *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
 		SheetSelection *ss = sheet_view->sheet->selections->data;
+		int col, distance;
 		
 		gtk_widget_destroy (gtk_widget_get_toplevel (sheet_view->tip));
 		sheet_view->tip = NULL;
 
-		sheet_cursor_move (sheet_view->sheet, GTK_ADJUSTMENT (sheet_view->ha)->value, ss->start_row);
+		col = GTK_ADJUSTMENT (sheet_view->ha)->value;
+
+		gnumeric_sheet_set_top_col (gsheet, col);
+		sheet_cursor_move (sheet_view->sheet, col, ss->start_row);
+		sheet_selection_append (sheet_view->sheet, col, ss->start_row);
 	}
 	
 	return FALSE;
@@ -386,13 +410,21 @@ vertical_scroll_event (GtkScrollbar *scroll, GdkEvent *event, SheetView *sheet_v
 		set_tip_label (sheet_view, _("Row: %d"), GTK_ADJUSTMENT (sheet_view->va), 0);
 		position_tooltip (sheet_view, 0);
 		gtk_widget_show_all (gtk_widget_get_toplevel (sheet_view->tip));
-	} else if (event->type == GDK_BUTTON_RELEASE){
+	}
+	else if (event->type == GDK_BUTTON_RELEASE)
+	{
+		GnumericSheet  *gsheet = GNUMERIC_SHEET (sheet_view->sheet_view);
 		SheetSelection *ss = sheet_view->sheet->selections->data;
+		int row, distance;
 		
 		gtk_widget_destroy (gtk_widget_get_toplevel (sheet_view->tip));
 		sheet_view->tip = NULL;
 
-		sheet_cursor_move (sheet_view->sheet, ss->start_col, GTK_ADJUSTMENT (sheet_view->va)->value);
+		row = GTK_ADJUSTMENT (sheet_view->va)->value;
+		
+		gnumeric_sheet_set_top_row (gsheet, row);
+		sheet_cursor_move (sheet_view->sheet, ss->start_col, row);
+		sheet_selection_append (sheet_view->sheet, ss->start_col, row);
 	}
 
 	return FALSE;
@@ -587,3 +619,72 @@ sheet_view_show_cursor (SheetView *sheet_view)
 
 	item_cursor_set_visibility (gsheet->item_cursor, TRUE);
 }
+
+#define TRIANGLE_WIDTH 6
+
+static GnomeCanvasPoints *
+sheet_view_comment_get_points (SheetView *sheet_view, int col, int row)
+{
+	GnomeCanvasPoints *points;
+	int x, y, i;
+
+	points = gnome_canvas_points_new (3);
+
+	x = sheet_col_get_distance (sheet_view->sheet, 0, col+1);
+	y = 1+sheet_row_get_distance (sheet_view->sheet, 0, row);
+
+	points->coords [0] = x - TRIANGLE_WIDTH;
+	points->coords [1] = y;
+	points->coords [2] = x;
+	points->coords [3] = y;
+	points->coords [4] = x;
+	points->coords [5] = y + TRIANGLE_WIDTH;
+
+	for (i = 0; i < 3; i++){
+		gnome_canvas_c2w (GNOME_CANVAS (sheet_view->sheet_view),
+				  points->coords [i*2],
+				  points->coords [i*2+1],
+				  &(points->coords [i*2]),
+				  &(points->coords [i*2+1]));
+	}
+	return points;
+}
+
+GnomeCanvasItem *
+sheet_view_comment_create_marker (SheetView *sheet_view, int col, int row)
+{
+	GnomeCanvasPoints *points;
+	GnomeCanvasGroup *group;
+	GnomeCanvasItem *i;
+	
+	g_return_val_if_fail (sheet_view != NULL, NULL);
+	g_return_val_if_fail (IS_SHEET_VIEW (sheet_view), NULL);
+
+	group = GNOME_CANVAS_GROUP (GNOME_CANVAS (sheet_view->sheet_view)->root);
+	points = sheet_view_comment_get_points (sheet_view, col, row);
+
+	i = gnome_canvas_item_new (
+		group, gnome_canvas_polygon_get_type (),
+		"points",     points,
+		"fill_color", "red",
+		NULL);
+
+	return i;
+}
+
+void
+sheet_view_comment_relocate (SheetView *sheet_view, int col, int row, GnomeCanvasItem *o)
+{
+	GnomeCanvasPoints *points;
+
+	g_return_if_fail (sheet_view != NULL);
+	g_return_if_fail (IS_SHEET_VIEW (sheet_view));
+	g_return_if_fail (o != NULL);
+	g_return_if_fail (GNOME_IS_CANVAS_ITEM (o));
+
+	points = sheet_view_comment_get_points (sheet_view, col, row);
+
+	gnome_canvas_item_set (o, "points", points, NULL);
+}
+
+
