@@ -1,5 +1,5 @@
-#ifndef SHEET_H
-#define SHEET_H
+#ifndef GNUMERIC_SHEET_H
+#define GNUMERIC_SHEET_H
 
 #define SHEET_MAX_ROWS (16 * 1024)
 #define SHEET_MAX_COLS 256
@@ -56,6 +56,9 @@ typedef struct {
 
 	gboolean   have_x_selection;
 
+	/* The Symbol table used for naming cell ranges in the workbook */
+	SymbolTable *symbol_names;
+	
 	/*
 	 * This is  used during the clipboard paste command to pass information
 	 * to the asyncronous paste callback
@@ -79,48 +82,50 @@ typedef enum {
 } SheetModeType;
 
 typedef struct {
-	int        signature;
+	int         signature;
 	
-	Workbook   *workbook;
-	GList      *sheet_views;
+	Workbook    *workbook;
+	GList       *sheet_views;
 	
-	char       *name;
+	char        *name;
 
-	GList      *style_list;	/* The list of styles applied to the sheets */
+	GList       *style_list;	/* The list of styles applied to the sheets */
 
-	ColRowInfo default_col_style;
-	GList      *cols_info;
+	ColRowInfo  default_col_style;
+	GList       *cols_info;
 
-	ColRowInfo default_row_style;
-	GList      *rows_info;
+	ColRowInfo  default_row_style;
+	GList       *rows_info;
 
-	GHashTable *cell_hash;	/* The cells in hashed format */
+	GHashTable  *cell_hash;	/* The cells in hashed format */
 
-	GList      *selections;
+	GList       *selections;
 
-	int        max_col_used;
-	int        max_row_used;
+	int         max_col_used;
+	int         max_row_used;
 
 	int         cursor_col, cursor_row;
 	
-	/* The list of formulas */
-	GList      *formula_cell_list;
-
-	double     last_zoom_factor_used;
+	/* The list of cells that have a comment */
+	GList       *comment_list;
+	
+	double      last_zoom_factor_used;
 
 	/* Objects */
 	SheetModeType mode;	/* Sheet mode */
-	GList      *objects;	/* List of objects in the spreadsheet */
-	GList      *coords;	/* During creation time: keeps click coordinates */
-	void       *current_object;
-	
+	GList       *objects;	/* List of objects in the spreadsheet */
+	GList       *coords;	/* During creation time: keeps click coordinates */
+	void        *current_object;
+
 	/*
 	 * When editing a cell: the cell (may be NULL) and
 	 * the original text of the cell
 	 */
-	String    *editing_saved_text;
-	Cell      *editing_cell;
-	int        editing;
+	String      *editing_saved_text;
+	Cell        *editing_cell;
+	int         editing;
+	
+	gboolean    modified;
 	
 	/* For walking trough a selection */
 	struct {
@@ -147,15 +152,20 @@ void        sheet_foreach_row          	 (Sheet *sheet,
 					  void *user_data);
 void        sheet_set_zoom_factor      	 (Sheet *sheet, double factor);
 void        sheet_cursor_set             (Sheet *sheet,
+					  int base_col,  int base_row,
 					  int start_col, int start_row,
 					  int end_col,   int end_row);
 void        sheet_cursor_move            (Sheet *sheet, int col, int row);
+void        sheet_make_cell_visible      (Sheet *sheet, int col, int row);
 
 /* Selection management */
 void        sheet_select_all             (Sheet *sheet);
 int         sheet_is_all_selected        (Sheet *sheet);
 void        sheet_selection_append       (Sheet *sheet, int col, int row);
 void        sheet_selection_extend_to    (Sheet *sheet, int col, int row);
+void	    sheet_selection_set		 (Sheet *sheet,
+					  int start_col, int start_row,
+					  int end_col, int end_row);
 void        sheet_selection_reset        (Sheet *sheet);
 void        sheet_selection_reset_only   (Sheet *sheet);
 int         sheet_selection_equal        (SheetSelection *a, SheetSelection *b);
@@ -166,16 +176,17 @@ void        sheet_selection_append_range (Sheet *sheet,
 CellList   *sheet_selection_to_list      (Sheet *sheet);
 
 /* Operations on the selection */
-void        sheet_selection_clear_content     (Sheet *sheet);
-void        sheet_selection_clear_formats     (Sheet *sheet);
 void        sheet_selection_clear             (Sheet *sheet);
+void        sheet_selection_clear_content     (Sheet *sheet);
+void        sheet_selection_clear_comments    (Sheet *sheet);
+void        sheet_selection_clear_formats     (Sheet *sheet);
 
 /* Cut/Copy/Paste on the workbook selection */
 gboolean    sheet_selection_copy              (Sheet *sheet);
 gboolean    sheet_selection_cut               (Sheet *sheet);
 void        sheet_selection_paste             (Sheet *sheet,
 					       int dest_col,    int dest_row,
-					       int paste_flags, guint32 time);
+					       int paste_flags, guint32 time32);
 int         sheet_selection_walk_step         (Sheet *sheet,
 					       int   forward,     int horizontal,
 					       int   current_col, int current_row,
@@ -200,6 +211,9 @@ int         sheet_cell_foreach_range      (Sheet *sheet, int only_existing,
 				           void *closure);
 Cell       *sheet_cell_get                (Sheet *sheet, int col, int row);
 Cell       *sheet_cell_fetch              (Sheet *sheet, int col, int row);
+void        sheet_cell_comment_link       (Cell *cell);
+void        sheet_cell_comment_unlink     (Cell *cell);
+
 void        sheet_cell_formula_link       (Cell *cell);
 void        sheet_cell_formula_unlink     (Cell *cell);
 
@@ -237,6 +251,9 @@ void        sheet_clear_region_formats    (Sheet *sheet,
 void        sheet_clear_region_content    (Sheet *sheet,
 				           int start_col, int start_row,
 				           int end_col,   int end_row);
+void        sheet_clear_region_comments   (Sheet *sheet,
+				           int start_col, int start_row,
+				           int end_col,   int end_row);	
 
 /* Sets the width/height of a column row in terms of pixels */
 void        sheet_col_set_width           (Sheet *sheet,
@@ -255,7 +272,9 @@ void        sheet_col_set_selection       (Sheet *sheet,
 void        sheet_row_set_selection       (Sheet *sheet,
 					   ColRowInfo *ri, int value);
 				       
-Style      *sheet_style_compute           (Sheet *sheet, int col, int row);
+Style      *sheet_style_compute           (Sheet *sheet,
+					   int col, int row,
+					   int *non_default_style_flags);
 
 /* Redraw */
 void        sheet_compute_visible_ranges  (Sheet *sheet);
@@ -267,6 +286,8 @@ void        sheet_redraw_all              (Sheet *sheet);
 				       
 void        sheet_update_auto_expr        (Sheet *sheet);
 
+void        sheet_mark_clean              (Sheet *sheet);
+
 /* Sheet information manipulation */
 void        sheet_insert_col              (Sheet *sheet,  int col, int count);
 void        sheet_delete_col              (Sheet *sheet,  int col, int count);
@@ -293,8 +314,9 @@ void        sheet_set_current_value       (Sheet *sheet);
 void        sheet_accept_pending_input    (Sheet *sheet);
 void        sheet_cancel_pending_input    (Sheet *sheet);
 void        sheet_load_cell_val           (Sheet *sheet);
-void        sheet_selection_col_extend_to (Sheet *sheet, int col);
-void        sheet_selection_row_extend_to (Sheet *sheet, int row);
+
+int         sheet_col_selection_type      (Sheet *sheet, int col);
+int         sheet_row_selection_type      (Sheet *sheet, int row);
 
 /*
  * Event state manipulation (for mode operation)
@@ -313,6 +335,11 @@ void        sheet_fill_selection_with     (Sheet *sheet, char *text);
 void        sheet_show_cursor             (Sheet *sheet);
 void        sheet_hide_cursor             (Sheet *sheet);
 
+char      *cellref_name                   (CellRef *cell_ref,
+					   Sheet *eval_sheet,
+					   int eval_col,
+					   int eval_row);
+
 /*
  * Workbook
  */
@@ -322,16 +349,20 @@ Workbook   *workbook_new_with_sheets     (int sheet_count);
 
 void        workbook_set_filename        (Workbook *, char *);
 void        workbook_set_title           (Workbook *, char *);
+Workbook   *workbook_read                (const char *filename);
 
 void        workbook_save_as             (Workbook *);
 void        workbook_save                (Workbook *);
 void        workbook_attach_sheet        (Workbook *, Sheet *);
 Sheet      *workbook_focus_current_sheet (Workbook *wb);
 Sheet      *workbook_get_current_sheet   (Workbook *wb);
+char       *workbook_sheet_get_free_name (Workbook *wb);
 void        workbook_auto_expr_label_set (Workbook *wb, char *text);
 void        workbook_next_generation     (Workbook *wb);
 void        workbook_set_region_status   (Workbook *wb, char *str);
 int         workbook_parse_and_jump      (Workbook *wb, char *text);
+Sheet      *workbook_sheet_lookup        (Workbook *wb, char *sheet_name);
+void        workbook_mark_clean          (Workbook *wb);
 
 /*
  * Does any pending recalculations
@@ -358,5 +389,5 @@ void     workbook_feedback_set        (Workbook *,
 				       void *data);
 
 extern   Workbook *current_workbook;
-#endif
+#endif /* GNUMERIC_SHEET_H */
 
