@@ -8,7 +8,6 @@
 #include <config.h>
 #include <gnome.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/lib_date.h>
 #include "gnumeric.h"
 #include "gnumeric-util.h"
 #include "gnumeric-sheet.h"
@@ -18,6 +17,9 @@
 #include "plugin.h"
 #include "pixmaps.h"
 
+#include "../plugins/excel/ms-ole.h"
+#include "../plugins/excel/ms-excel.h"
+
 /* The locations within the main table in the workbook */
 #define WB_EA_LINE   0
 #define WB_EA_SHEETS 1
@@ -28,26 +30,43 @@
 Workbook *current_workbook;
 static int workbook_count;
 
+static GList *workbook_list = NULL;
+
+/**
+ * Wrapper that decides which format to use
+ **/
+Workbook *
+workbook_read (const char *filename)
+{
+  /* A slow and possibly buggy check for now. */
+  MS_OLE *f = ms_ole_new (filename) ;
+  Workbook *wb;
+  if (f)
+    {
+      wb = ms_excelReadWorkbook(f) ;
+      free (f) ;
+    }
+  else
+    wb = gnumericReadXmlWorkbook (filename);
+  return wb ;
+}
+
 static void
 new_cmd (void)
 {
 	Workbook *wb;
-	
 	wb = workbook_new_with_sheets (1);
 	gtk_widget_show (wb->toplevel);
 }
 
 static void
-open_cmd (void)
+open_cmd (GtkWidget *widget, Workbook *wb)
 {
-	char *fname = dialog_query_load_file ();
-	Workbook *wb;
-	
-	if (!fname)
-		return;
-	wb = gnumericReadXmlWorkbook (fname);
-	if (wb)
-		gtk_widget_show (wb->toplevel);
+	char *fname = dialog_query_load_file (wb);
+	Workbook *new_wb;
+
+	if ((new_wb = workbook_read (fname)))
+	  gtk_widget_show (new_wb->toplevel);
 }
 
 static void
@@ -65,7 +84,7 @@ save_as_cmd (GtkWidget *widget, Workbook *wb)
 static void
 plugins_cmd (GtkWidget *widget, Workbook *wb)
 {
-	GtkWidget *pm = plugin_manager_new();
+	GtkWidget *pm = plugin_manager_new (wb);
 	gtk_widget_show(pm);
 }
 
@@ -142,8 +161,6 @@ change_selection_font (Workbook *wb, int idx, char *new[])
 		}
 	}
 
-	/* FIXME: Should attach the bold style here to the cell range */
-	
 	g_list_free (cells);
 }
 
@@ -210,15 +227,176 @@ create_ellipse_cmd (GtkWidget *widget, Workbook *wb)
 }
 
 static void
-quit_cmd (void)
+workbook_do_destroy (Workbook *wb)
 {
-	gtk_main_quit ();
+	if (wb->filename)
+	       g_free (wb->filename);
+
+	g_list_free (wb->eval_queue);
+
+	if (wb->auto_expr){
+		expr_tree_unref (wb->auto_expr);
+		string_unref (wb->auto_expr_desc);
+	}
+
+	workbook_list = g_list_remove (workbook_list, wb);
+	workbook_count--;
+
+	symbol_table_destroy (wb->symbol_names);
+	
+	if (workbook_count == 0)
+		gtk_main_quit ();
+}
+
+void
+workbook_destroy (Workbook *wb)
+{
+	gtk_widget_destroy (wb->toplevel);
+}
+
+static void
+workbook_widget_destroy (GtkWidget *widget, Workbook *wb)
+{
+	workbook_do_destroy (wb);
+}
+
+static void
+cb_sheet_mark_clean (gpointer key, gpointer value, gpointer user_data)
+{
+	Sheet *sheet = value;
+
+	sheet_mark_clean (sheet);
+}
+
+void
+workbook_mark_clean (Workbook *wb)
+{
+	g_return_if_fail (wb != NULL);
+
+	g_hash_table_foreach (wb->sheets, cb_sheet_mark_clean, NULL);
+}
+
+typedef enum {
+	CLOSE_DENY,
+	CLOSE_ALLOW,
+	CLOSE_RECHECK
+} CloseAction;
+
+static void
+cb_sheet_check_dirty (gpointer key, gpointer value, gpointer user_data)
+{
+	Sheet *sheet = value;
+	GtkWidget *d, *l;
+	int *allow_close = user_data;
+	int button;
+	char *f, *s;
+	
+	if (!sheet->modified)
+		return;
+
+	if (*allow_close != CLOSE_ALLOW)
+		return;
+	
+	d = gnome_dialog_new (
+		_("Warning"),
+		GNOME_STOCK_BUTTON_YES,
+		GNOME_STOCK_BUTTON_NO,
+		GNOME_STOCK_BUTTON_CANCEL,
+		NULL);
+
+	if (sheet->workbook->filename)
+		f = g_basename (sheet->workbook->filename);
+	else
+		f = "";
+	
+	s = g_copy_strings (_("Workbook "), f,
+			    _(" has unsaved changes, save them?"),
+			    NULL);
+	l = gtk_label_new (s);
+	gtk_widget_show (l);
+	g_free (s);
+	
+	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(d)->vbox), l, TRUE, TRUE, 0);
+
+	gtk_window_position (GTK_WINDOW (d), GTK_WIN_POS_MOUSE);
+	button = gnome_dialog_run (GNOME_DIALOG (d));
+
+	switch (button){
+		/* YES */
+	case 0:
+		workbook_save (sheet->workbook);
+		*allow_close = CLOSE_RECHECK;
+		break;
+
+		/* NO */
+	case 1:
+		workbook_mark_clean (sheet->workbook);
+		break;
+		
+		/* CANCEL */
+	case -1:
+	case 2:
+		*allow_close = CLOSE_DENY;
+		break;
+
+	} 
+
+	gtk_widget_destroy (d);
+}
+
+static int
+workbook_can_close (Workbook *wb)
+{
+	CloseAction allow_close;
+
+	do {
+		allow_close = CLOSE_ALLOW;
+		g_hash_table_foreach (
+			wb->sheets, cb_sheet_check_dirty, &allow_close);
+	} while (allow_close == CLOSE_RECHECK);
+
+	g_assert (allow_close == CLOSE_ALLOW || allow_close == CLOSE_DENY);
+	return allow_close == CLOSE_ALLOW;
 }
 
 static void
 close_cmd (GtkWidget *widget, Workbook *wb)
 {
-	workbook_destroy (wb);
+	if (workbook_can_close (wb))
+		workbook_destroy (wb);
+}
+
+static int
+workbook_delete_event (GtkWidget *widget, GdkEvent *event, Workbook *wb)
+{
+	if (workbook_can_close (wb)){
+		workbook_destroy (wb);
+		return TRUE;
+	} else
+		return FALSE;
+}
+
+static void
+quit_cmd (void)
+{
+	GList *l, *n = NULL;
+
+	/*
+	 * Duplicate the list as the workbook_list is modified during
+	 * workbook destruction
+	 */
+	
+	for (l = workbook_list; l; l = l->next)
+		n = g_list_prepend (n, l->data);
+	
+	for (l = n; l; l = l->next){
+		Workbook *wb = l->data;
+
+		if (workbook_can_close (wb))
+			workbook_destroy (wb);
+	}
+
+	g_list_free (n);
 }
 
 static void
@@ -264,7 +442,7 @@ paste_special_cmd (GtkWidget *widget, Workbook *wb)
 	int flags;
 
 	sheet = workbook_get_current_sheet (wb);
-	flags = dialog_paste_special ();
+	flags = dialog_paste_special (wb);
 	sheet_selection_paste (sheet, sheet->cursor_col, sheet->cursor_row,
 			       flags, GDK_CURRENT_TIME);
 	
@@ -285,13 +463,32 @@ goto_cell_cmd (GtkWidget *widget, Workbook *wb)
 	dialog_goto_cell (wb);
 }
 
+static void
+define_cell_cmd (GtkWidget *widget, Workbook *wb)
+{
+	dialog_define_names (wb);
+}
+
+static void
+insert_sheet_cmd (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet;
+	char *name;
+
+	name = workbook_sheet_get_free_name (wb);
+	sheet = sheet_new (wb, name);
+	g_free (name);
+	
+	workbook_attach_sheet (wb, sheet);
+}
+
 static void
 insert_cells_cmd (GtkWidget *widget, Workbook *wb)
 {
 	Sheet *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	dialog_insert_cells (sheet);
+	dialog_insert_cells (wb, sheet);
 }
 
 static void
@@ -344,6 +541,15 @@ clear_formats_cmd (GtkWidget *widget, Workbook *wb)
 	sheet_selection_clear_formats (sheet);
 }
 
+static void
+clear_comments_cmd (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet;
+
+	sheet = workbook_get_current_sheet (wb);
+	sheet_selection_clear_comments (sheet);
+}
+
 static void
 clear_content_cmd (GtkWidget *widget, Workbook *wb)
 {
@@ -359,7 +565,7 @@ zoom_cmd (GtkWidget *widget, Workbook *wb)
 	Sheet *sheet;
 
 	sheet = workbook_get_current_sheet (wb);
-	dialog_zoom (sheet);
+	dialog_zoom (wb, sheet);
 }
 
 static void
@@ -368,7 +574,7 @@ format_cells_cmd (GtkWidget *widget, Workbook *wb)
 	Sheet *sheet;
 	
 	sheet = workbook_get_current_sheet (wb);
-	dialog_cell_format (sheet);
+	dialog_cell_format (wb, sheet);
 }
 
 static void
@@ -396,13 +602,15 @@ static void
 insert_current_date_cmd (GtkWidget *widget, Workbook *wb)
 {
 	Sheet *sheet = workbook_get_current_sheet (wb);
-	time_t t = time (NULL);
-	struct tm *tm = localtime (&t);
 	char *preferred_date_format = _(">mm/dd/yyyy");
 	int n;
+	GDate *date = g_date_new();
 	
-	n = calc_days (tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday) -
-		calc_days (1900, 1, 1) + 1;
+	g_date_set_time (date, time (NULL));
+
+	n = g_date_serial (date);
+
+	g_date_free( date );
 
 	insert_at_cursor (sheet, n, preferred_date_format+1);
 }
@@ -420,81 +628,149 @@ insert_current_time_cmd (GtkWidget *widget, Workbook *wb)
 	insert_at_cursor (sheet, n, preferred_time_format);
 }
 
+static void
+workbook_edit_comment (GtkWidget *widget, Workbook *wb)
+{
+	Sheet *sheet = workbook_get_current_sheet (wb);
+	Cell *cell;
+	
+	cell = sheet_cell_get (sheet, sheet->cursor_col, sheet->cursor_row);
+
+	if (!cell){
+		cell = sheet_cell_new (sheet, sheet->cursor_col, sheet->cursor_row);
+		cell_set_text (cell, "");
+	}
+	
+	cell_set_comment (cell, "Test comment");
+}
+
 static void
 about_cmd (GtkWidget *widget, Workbook *wb)
 {
-	dialog_about ();
+	dialog_about (wb);
 }
 
+static void
+filenames_dropped(GtkWidget * widget,
+		  GdkDragContext   *context,
+		  gint              x,
+		  gint              y,
+		  GtkSelectionData *selection_data,
+		  guint             info,
+		  guint             time)
+{
+	GList *names, *tmp_list;
+  
+	names = gnome_uri_list_extract_filenames ((char *)selection_data->data);
+	tmp_list = names;
+	
+	while (tmp_list) {
+		Workbook *new_wb;
+		
+		if ((new_wb = workbook_read (tmp_list->data)))
+			gtk_widget_show (new_wb->toplevel);
+		
+		tmp_list = tmp_list->next;
+	}
+}
+
+/* File menu */
+
 static GnomeUIInfo workbook_menu_file [] = {
 	{ GNOME_APP_UI_ITEM, N_("_New"), NULL, new_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, GDK_n, GDK_CONTROL_MASK },
-	{ GNOME_APP_UI_ITEM, N_("_Open"), NULL, open_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, GDK_o, GDK_CONTROL_MASK },
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 'n', GDK_CONTROL_MASK },
+	{ GNOME_APP_UI_ITEM, N_("_Open..."), NULL, open_cmd, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 'o', GDK_CONTROL_MASK },
 	{ GNOME_APP_UI_ITEM, N_("_Save"), NULL, save_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, GDK_s, GDK_CONTROL_MASK },
-	{ GNOME_APP_UI_ITEM, N_("S_ave as..."), NULL, save_as_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE },
-	{ GNOME_APP_UI_ITEM, N_("_Plugins..."), NULL, plugins_cmd },
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 's', GDK_CONTROL_MASK },
+	{ GNOME_APP_UI_ITEM, N_("Save _as..."), NULL, save_as_cmd, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS },
+
+	GNOMEUIINFO_SEPARATOR,
+
+	{ GNOME_APP_UI_ITEM, N_("Plu_g-ins..."), NULL, plugins_cmd },
+
+	GNOMEUIINFO_SEPARATOR,
+
 	{ GNOME_APP_UI_ITEM, N_("_Close"), NULL, close_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, GDK_w, GDK_CONTROL_MASK },
-	{ GNOME_APP_UI_ITEM, N_("_Quit"), NULL, quit_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, GDK_q, GDK_CONTROL_MASK },
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 'w', GDK_CONTROL_MASK },
+	{ GNOME_APP_UI_ITEM, N_("E_xit"), NULL, quit_cmd, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', GDK_CONTROL_MASK },
 	GNOMEUIINFO_END
 };
 
+/* Edit menu */
+
 static GnomeUIInfo workbook_menu_edit_clear [] = {
-	{ GNOME_APP_UI_ITEM, N_("_All"),     NULL, clear_all_cmd },
-	{ GNOME_APP_UI_ITEM, N_("_Formats"), NULL, clear_formats_cmd },
-	{ GNOME_APP_UI_ITEM, N_("_Content"), NULL, clear_content_cmd },
+	{ GNOME_APP_UI_ITEM, N_("_All"),      NULL, clear_all_cmd },
+	{ GNOME_APP_UI_ITEM, N_("_Formats"),  NULL, clear_formats_cmd },
+	{ GNOME_APP_UI_ITEM, N_("_Comments"), NULL, clear_comments_cmd },
+	{ GNOME_APP_UI_ITEM, N_("_Content"),  NULL, clear_content_cmd },
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo workbook_menu_edit [] = {
-	{ GNOME_APP_UI_ITEM, N_("_Cut"), NULL, cut_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT, GDK_x, GDK_CONTROL_MASK },
-	{ GNOME_APP_UI_ITEM, N_("C_opy"), NULL, copy_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, GDK_c, GDK_CONTROL_MASK },
+	{ GNOME_APP_UI_ITEM, N_("Cu_t"), NULL, cut_cmd, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CUT, 'x', GDK_CONTROL_MASK },
+	{ GNOME_APP_UI_ITEM, N_("_Copy"), NULL, copy_cmd, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_COPY, 'c', GDK_CONTROL_MASK },
 	{ GNOME_APP_UI_ITEM, N_("_Paste"), NULL, paste_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE, GDK_v, GDK_CONTROL_MASK },
-	{ GNOME_APP_UI_ITEM, N_("P_aste special"), NULL, paste_special_cmd, NULL, NULL,
-	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE },
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PASTE, 'v', GDK_CONTROL_MASK },
+	{ GNOME_APP_UI_ITEM, N_("P_aste special..."), NULL, paste_special_cmd },
+	{ GNOME_APP_UI_SUBTREE, N_("C_lear"), NULL, workbook_menu_edit_clear },
+
+	GNOMEUIINFO_SEPARATOR,
+
 	{ GNOME_APP_UI_ITEM, N_("_Select All"), NULL, select_all_cmd, NULL, NULL,
-	  0, 0, GDK_a, GDK_CONTROL_MASK },
-	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_SUBTREE, N_("C_lear"), NULL, &workbook_menu_edit_clear },
-	GNOMEUIINFO_SEPARATOR,
+	  0, 0, 'a', GDK_CONTROL_MASK },
 	{ GNOME_APP_UI_ITEM, N_("_Goto cell.."), NULL, goto_cell_cmd, NULL, NULL,
-	  0, 0, GDK_i, GDK_CONTROL_MASK },
+	  0, 0, 'i', GDK_CONTROL_MASK },
+
 	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_ITEM, N_("Recalc"), NULL, recalc_cmd, NULL, NULL,
+
+#if 0
+	{ GNOME_APP_UI_ITEM, N_("_Define cell names"), NULL, define_cell_cmd },
+	GNOMEUIINFO_SEPARATOR,
+#endif
+	
+	{ GNOME_APP_UI_ITEM, N_("_Recalculate"), NULL, recalc_cmd, NULL, NULL,
 	  0, 0, GDK_F9, 0 },
 	GNOMEUIINFO_END
 };
 
+/* View menu */
+
 static GnomeUIInfo workbook_menu_view [] = {
 	{ GNOME_APP_UI_ITEM, N_("_Zoom..."), NULL, zoom_cmd },
 	GNOMEUIINFO_END
 };
 
+/* Insert menu */
+
 static GnomeUIInfo workbook_menu_insert_special [] = {
-	{ GNOME_APP_UI_ITEM, N_("Current date"), NULL, &insert_current_date_cmd,
+	{ GNOME_APP_UI_ITEM, N_("Current _date"), NULL, insert_current_date_cmd,
 	  NULL, NULL, 0, 0, ';', GDK_CONTROL_MASK },
-	{ GNOME_APP_UI_ITEM, N_("Current time"), NULL, &insert_current_time_cmd,
-	  NULL, NULL, 0, 0, ':', GDK_CONTROL_MASK },
+	{ GNOME_APP_UI_ITEM, N_("Current _time"), NULL, insert_current_time_cmd,
+	  NULL, NULL, 0, 0, ';', GDK_CONTROL_MASK | GDK_SHIFT_MASK },
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo workbook_menu_insert [] = {
+	{ GNOME_APP_UI_ITEM, N_("_Sheet"),    NULL, insert_sheet_cmd },
 	{ GNOME_APP_UI_ITEM, N_("_Cells..."), NULL, insert_cells_cmd },
-	{ GNOME_APP_UI_ITEM, N_("_Rows"),     NULL, insert_rows_cmd },
-	{ GNOME_APP_UI_ITEM, N_("C_olumns"),  NULL, insert_cols_cmd },
-	GNOMEUIINFO_SEPARATOR,
-	{ GNOME_APP_UI_SUBTREE, N_("Special"), NULL, &workbook_menu_insert_special },
+	{ GNOME_APP_UI_ITEM, N_("_Rows"),     NULL, insert_rows_cmd  },
+	{ GNOME_APP_UI_ITEM, N_("C_olumns"),  NULL, insert_cols_cmd  },
+
 	GNOMEUIINFO_SEPARATOR,
+
+	{ GNOME_APP_UI_ITEM, N_("_Add/Modify comment"), NULL, workbook_edit_comment },
+	{ GNOME_APP_UI_SUBTREE, N_("_Special"), NULL, workbook_menu_insert_special },
+
 	GNOMEUIINFO_END
 };
 
+/* Format menu */
+
 #if 0
 static GnomeUIInfo workbook_menu_format_column [] = {
 	{ GNOME_APP_UI_ITEM, N_("_Autoadjust"),   NULL, format_column_autoadjust_cmd },
@@ -515,29 +791,29 @@ static GnomeUIInfo workbook_menu_format_sheet [] = {
 #endif
 
 static GnomeUIInfo workbook_menu_format [] = {
-	{ GNOME_APP_UI_ITEM, N_("_Cells.."),   NULL, format_cells_cmd, NULL, NULL,
+	{ GNOME_APP_UI_ITEM, N_("_Cells..."),   NULL, format_cells_cmd, NULL, NULL,
 	  0, 0, GDK_1, GDK_CONTROL_MASK },
 #if 0
-	{ GNOME_APP_UI_SUBTREE, N_("C_olumn"), NULL, &workbook_menu_format_column },
-	{ GNOME_APP_UI_SUBTREE, N_("_Row"),    NULL, &workbook_menu_format_row },
-	{ GNOME_APP_UI_SUBTREE, N_("_Sheet"),  NULL, &workbook_menu_format_sheet },
+	{ GNOME_APP_UI_SUBTREE, N_("C_olumn"), NULL, workbook_menu_format_column },
+	{ GNOME_APP_UI_SUBTREE, N_("_Row"),    NULL, workbook_menu_format_row },
+	{ GNOME_APP_UI_SUBTREE, N_("_Sheet"),  NULL, workbook_menu_format_sheet },
 #endif
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo workbook_menu_help [] = {
-	{ GNOME_APP_UI_ITEM, N_("_About Gnumeric"), NULL, about_cmd, NULL, NULL,
+	{ GNOME_APP_UI_ITEM, N_("_About Gnumeric..."), NULL, about_cmd, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL },
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo workbook_menu [] = {
-	{ GNOME_APP_UI_SUBTREE, N_("_File"),   NULL, &workbook_menu_file },
-	{ GNOME_APP_UI_SUBTREE, N_("_Edit"),   NULL, &workbook_menu_edit },
-	{ GNOME_APP_UI_SUBTREE, N_("_View"),   NULL, &workbook_menu_view },
-	{ GNOME_APP_UI_SUBTREE, N_("_Insert"), NULL, &workbook_menu_insert },
-	{ GNOME_APP_UI_SUBTREE, N_("F_ormat"), NULL, &workbook_menu_format },
-	{ GNOME_APP_UI_SUBTREE, N_("_Help"),   NULL, &workbook_menu_help },
+	{ GNOME_APP_UI_SUBTREE, N_("_File"),   NULL, workbook_menu_file },
+	{ GNOME_APP_UI_SUBTREE, N_("_Edit"),   NULL, workbook_menu_edit },
+	{ GNOME_APP_UI_SUBTREE, N_("_View"),   NULL, workbook_menu_view },
+	{ GNOME_APP_UI_SUBTREE, N_("_Insert"), NULL, workbook_menu_insert },
+	{ GNOME_APP_UI_SUBTREE, N_("F_ormat"), NULL, workbook_menu_format },
+	{ GNOME_APP_UI_SUBTREE, N_("_Help"),   NULL, workbook_menu_help },
 	GNOMEUIINFO_END
 };
 
@@ -693,7 +969,8 @@ workbook_parse_and_jump (Workbook *wb, char *text)
 			gnumeric_notice (_("Row number out of range"));
 			return FALSE;
 		}
-			
+		
+		sheet_make_cell_visible (sheet, col, row);
 		sheet_cursor_move (sheet, col, row);
 		return TRUE;
 	}
@@ -791,7 +1068,7 @@ workbook_setup_edit_area (Workbook *wb)
 }
 
 
-struct {
+static struct {
 	char *displayed_name;
 	char *function;
 } quick_compute_routines [] = {
@@ -817,7 +1094,7 @@ workbook_set_auto_expr (Workbook *wb, char *description, char *expression)
 		expr_tree_unref (wb->auto_expr);
 		string_unref (wb->auto_expr_desc);
 	}
-	wb->auto_expr = expr_parse_string (expression, 0, 0, NULL, &error);
+	wb->auto_expr = expr_parse_string (expression, 0, 0, 0, NULL, &error);
 	wb->auto_expr_desc = string_get (description);
 
 	return error;
@@ -926,29 +1203,6 @@ workbook_set_focus (GtkWindow *window, GtkWidget *focus, Workbook *wb)
 		workbook_focus_current_sheet (wb);
 }
 
-void
-workbook_destroy (Workbook *wb)
-{
-	if (wb->filename)
-	       g_free (wb->filename);
-	g_list_free (wb->eval_queue);
-
-	gtk_widget_destroy (wb->toplevel);
-
-	if (wb->auto_expr){
-		expr_tree_unref (wb->auto_expr);
-		string_unref (wb->auto_expr_desc);
-	}
-}
-
-static void
-workbook_close (void)
-{
-	workbook_count--;
-	if (workbook_count == 0)
-		gtk_main_quit ();
-}
-
 /*
  * Sets up the workbook.
  * Right now it is adding some decorations to the window,
@@ -957,13 +1211,24 @@ workbook_close (void)
 Workbook *
 workbook_new (void)
 {
+	GnomeDockItem *item;
+	GtkWidget *toolbar;
+	
+	static GtkTargetEntry drag_types[] =
+	{
+		{ "text/uri-list", 0, 0 },
+	};
+	static gint n_drag_types = sizeof (drag_types) / sizeof (drag_types [0]);
+
 	Workbook *wb;
 
 	wb = g_new0 (Workbook, 1);
 	wb->toplevel  = gnome_app_new ("Gnumeric", "Gnumeric");
-	wb->sheets    = g_hash_table_new (g_str_hash, g_str_equal);
+	wb->sheets    = g_hash_table_new (gnumeric_strcase_hash, gnumeric_strcase_equal);
 	wb->table     = gtk_table_new (0, 0, 0);
 
+	wb->symbol_names = symbol_table_new ();
+
 	gtk_window_set_policy(GTK_WINDOW(wb->toplevel), 1, 1, 0);
 
 	wb->max_iterations = 1;
@@ -976,7 +1241,11 @@ workbook_new (void)
 	gnome_app_set_contents (GNOME_APP (wb->toplevel), wb->table);
 	gnome_app_create_menus_with_data (GNOME_APP (wb->toplevel), workbook_menu, wb);
 	gnome_app_create_toolbar_with_data (GNOME_APP (wb->toplevel), workbook_toolbar, wb);
-	gtk_toolbar_set_style (GTK_TOOLBAR (GNOME_APP (wb->toplevel)->toolbar), GTK_TOOLBAR_ICONS);
+
+	item = gnome_app_get_dock_item_by_name (GNOME_APP (wb->toplevel), GNOME_APP_TOOLBAR_NAME);
+	toolbar = gnome_dock_item_get_child (item);
+	
+	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
 
 	/* Focus handling */
 	gtk_signal_connect_after (
@@ -985,15 +1254,33 @@ workbook_new (void)
 
 	gtk_signal_connect (
 		GTK_OBJECT (wb->toplevel), "destroy",
-		GTK_SIGNAL_FUNC (workbook_close), wb);
+		GTK_SIGNAL_FUNC (workbook_widget_destroy), wb);
+
+	/* Enable toplevel as a drop target */
+
+	gtk_drag_dest_set (wb->toplevel,
+			   GTK_DEST_DEFAULT_ALL,
+			   drag_types, n_drag_types,
+			   GDK_ACTION_COPY);
+
+	gtk_signal_connect (GTK_OBJECT(wb->toplevel), 
+			    "drag_data_received",
+			    GTK_SIGNAL_FUNC(filenames_dropped), NULL);
 
 	/* clipboard setup */
 	x_clipboard_bind_workbook (wb);
 	
+	/* delete_event */
+	gtk_signal_connect (
+		GTK_OBJECT (wb->toplevel), "delete_event",
+		GTK_SIGNAL_FUNC (workbook_delete_event), wb);
+	
 	/* Set the default operation to be performed over selections */
 	workbook_set_auto_expr (wb, "SUM", "SUM(SELECTION())");
 
 	workbook_count++;
+
+	workbook_list = g_list_prepend (workbook_list, wb);
 	
 	gtk_widget_show_all (wb->table);
 	return wb;
@@ -1060,6 +1347,36 @@ workbook_attach_sheet (Workbook *wb, Sheet *sheet)
 				  t, gtk_label_new (sheet->name));
 }
 
+Sheet *
+workbook_sheet_lookup (Workbook *wb, char *sheet_name)
+{
+	Sheet *sheet;
+	
+	g_return_val_if_fail (wb != NULL, NULL);
+	g_return_val_if_fail (sheet_name != NULL, NULL);
+
+	sheet = g_hash_table_lookup (wb->sheets, sheet_name);
+
+	return sheet;
+}
+
+char *
+workbook_sheet_get_free_name (Workbook *wb)
+{
+	char name [80];
+	int  i;
+	
+	g_return_val_if_fail (wb != NULL, NULL);
+
+	for (i = 0; ; i++){
+		g_snprintf (name, sizeof (name), _("Sheet %d"), i);
+		if (workbook_sheet_lookup (wb, name) == NULL)
+			return g_strdup (name);
+	}
+	g_assert_not_reached ();
+	return NULL;
+}
+
 Workbook *
 workbook_new_with_sheets (int sheet_count)
 {
