@@ -3,7 +3,7 @@
  *
  * Daniel Veillard <Daniel.Veillard@w3.org>
  *
- * $Id: xml-io.c,v 1.24 1998/09/29 21:55:33 unammx Exp $
+ * $Id: xml-io.c,v 1.28 1998/11/10 20:53:00 unammx Exp $
  */
 
 #include <config.h>
@@ -46,20 +46,26 @@ static void nameFree                (gpointer key, gpointer value,
  * Get a value for a node either carried as an attibute or as
  * the content of a child.
  */
-static const char *
+static char *
 xmlGetValue (xmlNodePtr node, const char *name)
 {
-	const char *ret;
+	char *ret;
 	xmlNodePtr child;
 
-	ret = xmlGetProp (node, name);
+	ret = (char *) xmlGetProp (node, name);
 	if (ret != NULL)
-		return ret;
+		return strdup (ret);
 	child = node->childs;
 
 	while (child != NULL) {
-		if ((!strcmp (child->name, name)) && (child->content != NULL))
-			return child->content;
+		if (!strcmp (child->name, name)) {
+		        /*
+			 * !!! Inefficient, but ...
+			 */
+			ret = xmlNodeGetContent(child);
+			if (ret != NULL)
+			    return (ret);
+		}
 		child = child->next;
 	}
 
@@ -70,28 +76,17 @@ xmlGetValue (xmlNodePtr node, const char *name)
  * Get a String value for a node either carried as an attibute or as
  * the content of a child.
  */
-String *
+static String *
 xmlGetStringValue (xmlNodePtr node, const char *name)
 {
-	const char *val;
+	char *val;
 	String *ret;
-	xmlNodePtr child;
 
-	val = xmlGetProp (node, name);
-	
-	if (val != NULL){
-		ret = string_get ((char *) val);
-		return ret;
-	}
-	child = node->childs;
-	while (child != NULL){
-		if ((!strcmp (child->name, name)) && (child->content != NULL)) {
-			ret = string_get ((char *) child->content);
-			return ret;
-		}
-		child = child->next;
-	}
-	return NULL;
+	val = xmlGetValue(node, name);
+	if (val == NULL) return(NULL);
+        ret = string_get(val);
+	free(val);
+	return(ret);
 }
 
 /*
@@ -101,24 +96,19 @@ xmlGetStringValue (xmlNodePtr node, const char *name)
 static int
 xmlGetIntValue (xmlNodePtr node, const char *name, int *val)
 {
-	const char *ret;
-	xmlNodePtr child;
+	char *ret;
 	int i;
+	int res;
 
-	ret = xmlGetProp (node, name);
-	if ((ret != NULL) && (sscanf (ret, "%d", &i) == 1)){
-		*val = i;
+	ret = xmlGetValue (node, name);
+	if (ret == NULL) return(0);
+	res = sscanf (ret, "%d", &i);
+	free(ret);
+	
+	if (res == 1) {
+	        *val = i;
 		return 1;
 	}
-	child = node->childs;
-	while (child != NULL){
-		if (((!strcmp (child->name, name)) && (child->content != NULL) &&
-		     (sscanf (child->content, "%d", &i) == 1))){
-			*val = i;
-			return 1;
-		}
-		child = child->next;
-	}
 	return 0;
 }
 
@@ -126,27 +116,23 @@ xmlGetIntValue (xmlNodePtr node, const char *name, int *val)
  * Get a float value for a node either carried as an attibute or as
  * the content of a child.
  */
-int
+static int
 xmlGetFloatValue (xmlNodePtr node, const char *name, float *val)
 {
-	const char *ret;
+	int res;
+	char *ret;
 	xmlNodePtr child;
 	float f;
 
-	ret = xmlGetProp (node, name);
-	if ((ret != NULL) && (sscanf (ret, "%f", &f) == 1)){
-		*val = f;
+	ret = xmlGetValue (node, name);
+	if (ret == NULL) return(0);
+	res = sscanf (ret, "%f", &f);
+	free(ret);
+	
+	if (res == 1) {
+	        *val = f;
 		return 1;
 	}
-	child = node->childs;
-	while (child != NULL){
-		if (((!strcmp (child->name, name)) && (child->content != NULL) &&
-		     (sscanf (child->content, "%f", &f) == 1))){
-			*val = f;
-			return 1;
-		}
-		child = child->next;
-	}
 	return 0;
 }
 
@@ -157,24 +143,20 @@ xmlGetFloatValue (xmlNodePtr node, const char *name, float *val)
 static int
 xmlGetDoubleValue (xmlNodePtr node, const char *name, double *val)
 {
-	const char *ret;
+	int res;
+	char *ret;
 	xmlNodePtr child;
 	float f;
 
-	ret = xmlGetProp (node, name);
-	if ((ret != NULL) && (sscanf (ret, "%f", &f) == 1)){
-		*val = f;
+	ret = xmlGetValue (node, name);
+	if (ret == NULL) return(0);
+	res = sscanf (ret, "%f", &f);
+	free(ret);
+	
+	if (res == 1) {
+	        *val = f;
 		return 1;
 	}
-	child = node->childs;
-	while (child != NULL){
-		if (((!strcmp (child->name, name)) && (child->content != NULL) &&
-		     (sscanf (child->content, "%f", &f) == 1))){
-			*val = f;
-			return 1;
-		}
-		child = child->next;
-	}
 	return 0;
 }
 
@@ -182,24 +164,24 @@ xmlGetDoubleValue (xmlNodePtr node, const char *name, double *val)
  * Get a set of coodinates for a node, carried as the content of a child.
  */
 
-int
+static int
 xmlGetCoordinate (xmlNodePtr node, const char *name,
 		  double *x, double *y)
 {
+	int res;
+	char *ret;
 	xmlNodePtr child;
 	float X, Y;
 
-	child = node->childs;
-	while (child != NULL){
-		if ((!strcmp (child->name, name)) && (child->content != NULL)){
-			if (sscanf (child->content, "(%f %f)", &X, &Y) == 2){
-				*x = X;
-				*y = Y;
-				return 1;
-			}
-			return 0;
-		}
-		child = child->next;
+	ret = xmlGetValue (node, name);
+	if (ret == NULL) return(0);
+	res = sscanf (ret, "(%f %f)", &X, &Y);
+	free(ret);
+	
+	if (res == 2) {
+		*x = X;
+		*y = Y;
+		return 1;
 	}
 	return 0;
 }
@@ -212,23 +194,22 @@ static int
 xmlGetCoordinates (xmlNodePtr node, const char *name,
 		   double *x1, double *y1, double *x2, double *y2)
 {
+	int res;
+	char *ret;
 	xmlNodePtr child;
 	float X1, Y1, X2, Y2;
 
-	child = node->childs;
-	while (child != NULL){
-		if ((!strcmp (child->name, name)) && (child->content != NULL)){
-			if (sscanf (child->content, "(%f %f)(%f %f)",
-				    &X1, &Y1, &X2, &Y2) == 4){
-				*x1 = X1;
-				*y1 = Y1;
-				*x2 = X2;
-				*y2 = Y2;
-				return 1;
-			}
-			return 0;
-		}
-		child = child->next;
+	ret = xmlGetValue (node, name);
+	if (ret == NULL) return(0);
+	res = sscanf (ret, "(%f %f)(%f %f)", &X1, &Y1, &X2, &Y2);
+	free(ret);
+	
+	if (res == 2) {
+		*x1 = X1;
+		*y1 = Y1;
+		*x2 = X2;
+		*y2 = Y2;
+		return 1;
 	}
 	return 0;
 }
@@ -237,9 +218,10 @@ xmlGetCoordinates (xmlNodePtr node, const char *name,
  * Get a GnomeCanvasPoints for a node, carried as the content of a child.
  */
 
-GnomeCanvasPoints *
+static GnomeCanvasPoints *
 xmlGetGnomeCanvasPoints (xmlNodePtr node, const char *name)
 {
+	char *val;
 	GnomeCanvasPoints *ret = NULL;
 	int res;
 	const char *ptr;
@@ -247,31 +229,28 @@ xmlGetGnomeCanvasPoints (xmlNodePtr node, const char *name)
 	int index = 0, i;
 	float coord[20];	/* TODO: must be dynamic !!!! */
 
-	child = node->childs;
-	while (child != NULL){
-		if ((!strcmp (child->name, name)) && (child->content != NULL)){
-			ptr = child->content;
-			do {
-				while ((*ptr) && (*ptr != '('))
-					ptr++;
-				if (*ptr == 0)
-					break;
-				res = sscanf (ptr, "(%f %f)", &coord[index], &coord[index + 1]);
-				if (res != 2)
-					break;
-				index += 2;
-				ptr++;
-			} while (res > 0);
-			if (index >= 2)
-				ret = gnome_canvas_points_new (index / 2);
-			if (ret == NULL)
-				return NULL;
-			for (i = 0; i < index; i++)
-				ret->coords[i] = coord[i];
+	val = xmlGetValue (node, name);
+	if (ret == NULL) return(NULL);
+	ptr = val;
+	do {
+		while ((*ptr) && (*ptr != '('))
+			ptr++;
+		if (*ptr == 0)
 			break;
-		}
-		child = child->next;
-	}
+		res = sscanf (ptr, "(%f %f)", &coord[index], &coord[index + 1]);
+		if (res != 2)
+			break;
+		index += 2;
+		ptr++;
+	} while (res > 0);
+	free(val);
+
+	if (index >= 2)
+		ret = gnome_canvas_points_new (index / 2);
+	if (ret == NULL)
+		return NULL;
+	for (i = 0; i < index; i++)
+		ret->coords[i] = coord[i];
 	return ret;
 }
 
@@ -308,7 +287,7 @@ xmlSetGnomeCanvasPoints (xmlNodePtr node, const char *name,
 		}
 		child = child->next;
 	}
-	xmlNewChild (node, NULL, name, base);
+	xmlNewChild (node, NULL, name, xmlEncodeEntities(node->doc, base));
 	free (base);
 }
 
@@ -395,7 +374,8 @@ xmlSetIntValue (xmlNodePtr node, const char *name, int val)
  * Set a float value for a node either carried as an attibute or as
  * the content of a child.
  */
-void xmlSetFloatValue (xmlNodePtr node, const char *name, float val)
+static void
+xmlSetFloatValue (xmlNodePtr node, const char *name, float val)
 {
 	const char *ret;
 	xmlNodePtr child;
@@ -422,7 +402,8 @@ void xmlSetFloatValue (xmlNodePtr node, const char *name, float val)
  * Set a double value for a node either carried as an attibute or as
  * the content of a child.
  */
-void xmlSetDoubleValue (xmlNodePtr node, const char *name, double val)
+static void
+xmlSetDoubleValue (xmlNodePtr node, const char *name, double val)
 {
 	const char *ret;
 	xmlNodePtr child;
@@ -481,25 +462,17 @@ static int
 xmlGetColorValue (xmlNodePtr node, const char *name,
 			     StyleColor **color)
 {
-	const char *ret;
+	char *ret;
 	xmlNodePtr child;
 	int red, green, blue;
 
-	ret = xmlGetProp (node, name);
-	if ((ret != NULL) &&
-	    (sscanf (ret, "%X:%X:%X", &red, &green, &blue) == 3)){
+	ret = xmlGetValue (node, name);
+	if (ret == NULL) return(0);
+	if (sscanf (ret, "%X:%X:%X", &red, &green, &blue) == 3){
 		*color = style_color_new (red, green, blue);
+		free(ret);
 		return 1;
 	}
-	child = node->childs;
-	while (child != NULL){
-		if ((!strcmp (child->name, name)) && (child->content != NULL) &&
-		  (sscanf (ret, "%X:%X:%X", &red, &green, &blue) == 3)){
-			*color = style_color_new (red, green, blue);
-			return 1;
-		}
-		child = child->next;
-	}
 	return 0;
 }
 
@@ -582,6 +555,8 @@ Sheet *gnumericReadXmlSheet (const char *filename)
 	g_hash_table_destroy (ctxt.nameTable);
 
 	xmlFreeDoc (res);
+
+	sheet->modified = FALSE;
 	return sheet;
 }
 
@@ -625,6 +600,7 @@ gnumericWriteXmlSheet (Sheet * sheet, const char *filename)
 	xmlSetDocCompressMode (xml, 9);
 	ret = xmlSaveFile (filename, xml);
 	xmlFreeDoc (xml);
+	sheet->modified = FALSE;
 	if (ret < 0)
 		return -1;
 	return 0;
@@ -671,7 +647,7 @@ Workbook *gnumericReadXmlWorkbook (const char *filename)
 	ctxt.nameTable = g_hash_table_new (g_str_hash, g_str_equal);
 	ctxt.fontIdx = 1;
 	wb = readXmlWorkbook (&ctxt, res->root);
-	workbook_set_filename (wb, filename);
+	workbook_set_filename (wb, (char *) filename);
 	workbook_recalc_all (wb);
 	g_hash_table_foreach (ctxt.nameTable, nameFree, NULL);
 	g_hash_table_destroy (ctxt.nameTable);
@@ -787,7 +763,7 @@ writeXmlStyleBorder (parseXmlContextPtr ctxt, StyleBorder *border)
 	if ((border->left == BORDER_NONE) && (border->right == BORDER_NONE) &&
 	 (border->left == BORDER_NONE) && (border->right == BORDER_NONE))
 		return NULL;
-	cur = xmlNewNode (ctxt->ns, "StyleBorder", NULL);
+	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "StyleBorder", NULL);
 	if (border->left != BORDER_NONE){
 		side = xmlNewChild (cur, ctxt->ns, "Left", BorderTypes[border->left]);
 		xmlSetColorValue (side, "Color", border->left_color);
@@ -865,7 +841,7 @@ writeXmlStyle (parseXmlContextPtr ctxt, Style * style)
 	    (style->font == NULL) && (style->border == NULL))
 		return NULL;
 
-	cur = xmlNewNode (ctxt->ns, "Style", NULL);
+	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Style", NULL);
 	xmlSetIntValue (cur, "HAlign", style->halign);
 	xmlSetIntValue (cur, "VAlign", style->valign);
 	xmlSetIntValue (cur, "Fit", style->fit_in_cell);
@@ -889,7 +865,9 @@ writeXmlStyle (parseXmlContextPtr ctxt, Style * style)
 			sprintf (str, "#%s", name);
 			xmlNewProp (child, "HREF", str);
 		} else {
-			child = xmlNewChild (cur, ctxt->ns, "Font", style->font->font_name);
+			child = xmlNewChild (cur, ctxt->ns, "Font", 
+			           xmlEncodeEntities(ctxt->doc,
+				                     style->font->font_name));
 			xmlSetIntValue (child, "Unit", style->font->units);
 			sprintf (str, "FontDef%d", ctxt->fontIdx++);
 			xmlNewProp (child, "NAME", str);
@@ -920,7 +898,7 @@ readXmlStyle (parseXmlContextPtr ctxt, xmlNodePtr tree, Style * ret)
 
 	if (strcmp (tree->name, "Style")){
 		fprintf (stderr,
-			 "readXmlStyle: invalid element type %s, 'Style' expected`\n",
+			 "readXmlStyle: invalid element type %s, 'Style' expected\n",
 			 tree->name);
 	}
 	if (ret == NULL)
@@ -973,9 +951,14 @@ readXmlStyle (parseXmlContextPtr ctxt, xmlNodePtr tree, Style * ret)
 			v = xmlGetValue (child, "NAME");
 			if (v){
 				int units = 14;
+				char *font;
 
 				xmlGetIntValue (child, "Unit", &units);
-				ret->font = style_font_new (child->content, units);
+				font = xmlNodeGetContent(child);
+				if (font != NULL) {
+					ret->font = style_font_new (font, units);
+					free(font);
+				}
 				if (ret->font){
 					g_hash_table_insert (ctxt->nameTable, g_strdup (v), ret->font);
 					ret->valid_flags |= STYLE_FONT;
@@ -1011,13 +994,12 @@ readXmlStyle (parseXmlContextPtr ctxt, xmlNodePtr tree, Style * ret)
 /*
  * Create an XML subtree of doc equivalent to the given StyleRegion.
  */
-/* static !!! */
-xmlNodePtr
+static xmlNodePtr
 writeXmlStyleRegion (parseXmlContextPtr ctxt, StyleRegion *region)
 {
 	xmlNodePtr cur, child;
 
-	cur = xmlNewNode (ctxt->ns, "StyleRegion", NULL);
+	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "StyleRegion", NULL);
 	xmlSetIntValue (cur, "startCol", region->range.start_col);
 	xmlSetIntValue (cur, "endCol", region->range.end_col);
 	xmlSetIntValue (cur, "startRow", region->range.start_row);
@@ -1069,9 +1051,9 @@ writeXmlColRowInfo (parseXmlContextPtr ctxt, ColRowInfo *info, int col)
 	xmlNodePtr cur;
 
 	if (col)
-		cur = xmlNewNode (ctxt->ns, "ColInfo", NULL);
+		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "ColInfo", NULL);
 	else
-		cur = xmlNewNode (ctxt->ns, "RowInfo", NULL);
+		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "RowInfo", NULL);
 
 	xmlSetIntValue (cur, "No", info->pos);
 	xmlSetIntValue (cur, "Unit", info->units);
@@ -1133,7 +1115,7 @@ writeXmlObject (parseXmlContextPtr ctxt, SheetObject *object)
 	case SHEET_OBJECT_RECTANGLE:{
 			SheetFilledObject *fo = (SheetFilledObject *) object;
 
-			cur = xmlNewNode (ctxt->ns, "Rectangle", NULL);
+			cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Rectangle", NULL);
 			if (fo->fill_color != NULL)
 				xmlSetStringValue (cur, "FillColor", fo->fill_color);
 			xmlSetIntValue (cur, "Pattern", fo->pattern);
@@ -1142,17 +1124,17 @@ writeXmlObject (parseXmlContextPtr ctxt, SheetObject *object)
 	case SHEET_OBJECT_ELLIPSE:{
 			SheetFilledObject *fo = (SheetFilledObject *) object;
 
-			cur = xmlNewNode (ctxt->ns, "Ellipse", NULL);
+			cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Ellipse", NULL);
 			if (fo->fill_color != NULL)
 				xmlSetStringValue (cur, "FillColor", fo->fill_color);
 			xmlSetIntValue (cur, "Pattern", fo->pattern);
 			break;
 		}
 	case SHEET_OBJECT_ARROW:
-		cur = xmlNewNode (ctxt->ns, "Arrow", NULL);
+		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Arrow", NULL);
 		break;
 	case SHEET_OBJECT_LINE:
-		cur = xmlNewNode (ctxt->ns, "Line", NULL);
+		cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Line", NULL);
 		break;
 	}
 	if (cur == NULL)
@@ -1222,14 +1204,18 @@ static xmlNodePtr
 writeXmlCell (parseXmlContextPtr ctxt, Cell *cell)
 {
 	xmlNodePtr cur, child;
-
-	cur = xmlNewNode (ctxt->ns, "Cell", cell->entered_text->str);
+	char *text;
+	
+	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Cell", NULL);
 	xmlSetIntValue (cur, "Col", cell->col->pos);
 	xmlSetIntValue (cur, "Row", cell->row->pos);
 	child = writeXmlStyle (ctxt, cell->style);
 
 	if (child)
 		xmlAddChild (cur, child);
+	text = cell_get_text (cell);
+	xmlNewChild(cur, ctxt->ns, "Content", text);
+	g_free (text);
 
 	return cur;
 }
@@ -1242,11 +1228,13 @@ readXmlCell (parseXmlContextPtr ctxt, xmlNodePtr tree)
 {
 	Style *style;
 	Cell *ret;
+	xmlNodePtr childs;
 	int row = 0, col = 0;
+	char *content = NULL;
 
 	if (strcmp (tree->name, "Cell")){
 		fprintf (stderr,
-			 "readXmlCell: invalid element type %s, 'Cell' expected`\n",
+		 "readXmlCell: invalid element type %s, 'Cell' expected`\n",
 			 tree->name);
 		return NULL;
 	}
@@ -1259,25 +1247,34 @@ readXmlCell (parseXmlContextPtr ctxt, xmlNodePtr tree)
 	if (ret == NULL)
 		return NULL;
 
-	style = readXmlStyle (ctxt, tree->childs, NULL);
-	if (style){
-		style_merge_to (style, ret->style);
-		style_destroy (ret->style);
-		ret->style = style;
+	childs = tree->childs;
+	while (childs != NULL) {
+	        if (!strcmp (childs->name, "Style")) {
+		        style = readXmlStyle (ctxt, childs, NULL);
+			if (style){
+				style_merge_to (style, ret->style);
+				style_destroy (ret->style);
+				ret->style = style;
+			}
+		}
+	        if (!strcmp (childs->name, "Content"))
+		        content = xmlNodeGetContent(childs);
+		childs = childs->next;
 	}
-	if (tree->content != NULL){
-		char *v = g_strdup (tree->content);
-		char *p = v + strlen (v);
+	if (content == NULL)
+		content = xmlNodeGetContent(tree);
+	if (content != NULL) {
+		char *p = content + strlen (content);
 
-		while (p > v){
+		while (p > content){
 			p--;
 			if (*p != ' ' && *p != '\n')
 				break;
 			*p = 0;
 		}
 
-		cell_set_text_simple (ret, v);
-		g_free (v);
+		cell_set_text_simple (ret, content);
+		free (content);
 	} else
 		cell_set_text_simple (ret, "");
 
@@ -1315,10 +1312,11 @@ writeXmlSheet (parseXmlContextPtr ctxt, Sheet *sheet)
 	/*
 	 * General informations about the Sheet.
 	 */
-	cur = xmlNewNode (ctxt->ns, "Sheet", NULL);
+	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Sheet", NULL);
 	if (cur == NULL)
 		return NULL;
-	xmlNewChild (cur, ctxt->ns, "Name", sheet->name);
+	xmlNewChild (cur, ctxt->ns, "Name", 
+	             xmlEncodeEntities(ctxt->doc, sheet->name));
 	sprintf (str, "%d", sheet->max_col_used);
 	xmlNewChild (cur, ctxt->ns, "MaxCol", str);
 	sprintf (str, "%d", sheet->max_row_used);
@@ -1380,6 +1378,7 @@ writeXmlSheet (parseXmlContextPtr ctxt, Sheet *sheet)
 	cells = xmlNewChild (cur, ctxt->ns, "Cells", NULL);
 	ctxt->parent = cells;
 	g_hash_table_foreach (sheet->cell_hash, writeXmlCellTo, ctxt);
+	sheet->modified = 0;
 	return cur;
 }
 
@@ -1506,7 +1505,7 @@ writeXmlWorkbook (parseXmlContextPtr ctxt, Workbook *wb)
 	/*
 	 * General informations about the Sheet.
 	 */
-	cur = xmlNewNode (ctxt->ns, "Workbook", NULL);	/* the Workbook name !!! */
+	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "Workbook", NULL);	/* the Workbook name !!! */
 	if (cur == NULL)
 		return NULL;
 
@@ -1514,7 +1513,7 @@ writeXmlWorkbook (parseXmlContextPtr ctxt, Workbook *wb)
 	if (child)
 		xmlAddChild (cur, child);
 
-	child = xmlNewNode (ctxt->ns, "Geometry", NULL);
+	child = xmlNewDocNode (ctxt->doc, ctxt->ns, "Geometry", NULL);
 	xmlSetIntValue (child, "Width", wb->toplevel->allocation.width);
 	xmlSetIntValue (child, "Height", wb->toplevel->allocation.height);
 	xmlAddChild (cur, child);
