@@ -3,15 +3,15 @@
  *
  * Daniel Veillard <Daniel.Veillard@w3.org>
  *
- * $Id: xml-io.h,v 1.2 1998/08/31 06:49:57 unammx Exp $
+ * $Id: xml-io.h,v 1.4 1998/11/10 20:53:01 unammx Exp $
  */
 
-#ifndef __GNUMERIC_XML_IO__
-#define __GNUMERIC_XML_IO__
+#ifndef GNUMERIC_XML_IO_H
+#define GNUMERIC_XML_IO_H
 
-Sheet*    gnumericReadXmlSheet     (const char *filename);
+Sheet    *gnumericReadXmlSheet     (const char *filename);
 int       gnumericWriteXmlSheet    (Sheet *sheet, const char *filename);
-Workbook* gnumericReadXmlWorkbook  (const char *filename);
+Workbook *gnumericReadXmlWorkbook  (const char *filename);
 int       gnumericWriteXmlWorkbook (Workbook *sheet, const char *filename);
 
-#endif /* __GNUMERIC_XML_IO__ */
+#endif /* GNUMERIC_XML_IO_H */
